::--------------------------------------------------------------
::
:: File: GenerateDocumentation.cmd
:: 
:: Generate Doxygen documentation for a SW project
:: Syntax: GenerateDocumentation [ProjectPath] [ProjectName]
::
:: Remark: Syntax when used as external tool within Visual Studio:
:: GenerateDocumentation $(ProjectDir) $(ProjectFileName)
::
::--------------------------------------------------------------

@echo off
setlocal

set thisFile=GenerateDocumentation.cmd
set relativePathToDoxyGenFiles=BuildAll
set nameGenericDoxyFile=DoxygenTemplateForDocGeneration.dxy
set nameSpecificDoxyFile=GeneratedDoxyFileDoNotEditManually.dxy
set nameDoxygenOutputDir=DoxyGenDoc

::-----  Command line params  -----

echo Generating DoxyGen documentation... (%thisFile%))
echo.

set inputProjectDir=%~1%
set inputProjectName=%~2%

echo ProjectDir from cmd line   : %inputProjectDir%
echo ProjectName from cmd line  : %inputProjectName%

if "%inputProjectDir%" == "" goto :ERR_MISSING_CMDLINE_PARAMS
if "%inputProjectName%" == "" goto :ERR_MISSING_CMDLINE_PARAMS

:: cut trailing "\\"
set projectDir=%inputProjectDir:\\=%

:: cut extension from project file name
for /f "tokens=1,2 delims=/." %%a in ("%inputProjectName%") do set projectName=%%a

echo ProjectDir                 : %projectDir%
echo ProjectName                : %projectName%


::-----  Get specific environment paths  -----

set sourceDir=%MY_SOURCE_DIR%
:: cut trailing "\\"
set sourceDir=%sourceDir:\\=%
if "%sourceDir%" == "" goto :ERR_SOURCE_DIR_NOT_SET
set sourceDrive=%sourceDir:~0,2%
echo SourceDir from env         : %sourceDir%

set targetDir=C:\MyDocTarget
::set targetDir=%MY_TARGET_DIR%
:: cut trailing "\\"
set targetDir=%targetDir:\\=%
if "%targetDir%" == "" goto :ERR_TARGET_DIR_NOT_SET
set targetDrive=%targetDir:~0,2%
echo TargetDir from env         : %targetDir%

echo SourceDrive                : %sourceDrive%
echo TargetDrive                : %targetDrive%

%targetDrive%
cd %targetDir%
if not exist %nameDoxygenOutputDir% (
mkdir %nameDoxygenOutputDir%
)
set targetDir=%targetDir%\%nameDoxygenOutputDir%
if not exist %targetDir% goto :ERR_TARGET_DIR_NOT_CREATABLE
cd %targetDir%
if not exist %projectName% (
mkdir %projectName%
)
set targetDir=%targetDir%\%projectName%
echo TargetDir                  : %targetDir%
if not exist %targetDir% goto :ERR_TARGET_DIR_NOT_CREATABLE


::-----  Build names for doxy files  -----

set doxyTemplatePath=%sourceDir%\%relativePathToDoxyGenFiles%\%nameGenericDoxyFile%
:: replace "\\" with "\"
set doxyTemplatePath=%doxyTemplatePath:\\=\%
echo Generic doxy template      : %doxyTemplatePath%

set nameSpecificDoxyFile=%targetDir%\%nameSpecificDoxyFile%
:: replace "\\" with "\"
set nameSpecificDoxyFile=%nameSpecificDoxyFile:\\=\%
echo Specific doxy file         : %nameSpecificDoxyFile%


::-----  Generating specific doxy file  -----

echo.
echo Generating specific doxy file %nameSpecificDoxyFile% ...
set d="%nameSpecificDoxyFile%"
echo # Automatically generated file > %d%
echo # DO NOT EDIT MANUALLY! >> %d%
echo. >> %d%
echo # BEGIN ------- Contents of generic doxy file >> %d%
type %doxyTemplatePath% >> %d%
echo # END   ------- Contents of generic doxy file >> %d%
echo. >> %d%
echo # BEGIN ------- Project specific settings >> %d%
echo PROJECT_NAME           = "%projectName%" >> %d%
echo OUTPUT_DIRECTORY       = "%targetDir%" >> %d%
echo # END  -------- Project specific settings >> %d%


::-----  Generating docoment  -----

pushd .

echo.
echo ----- doc generation in progress ...
echo.
echo scanning for contents in directory:
%sourceDrive%
cd %projectDir%
cd

echo.
echo calling doxygen %nameSpecificDoxyFile% ...
doxygen.exe "%nameSpecificDoxyFile%"
echo.
echo ----- doc generation finished  -----
popd
endlocal
goto :eof

:ERR_MISSING_CMDLINE_PARAMS
echo ERROR: Expected calling syntax with params: projectDir projectName
echo Hint: When called from Visual Studio Tools menu you can use: $(ProjectDir) $(ProjectFileName)
goto :TERMINATE_WITH_ERROR

:ERR_SOURCE_DIR_NOT_SET
echo ERROR: Source dir MY_SOURCE_DIR not set as environment variable!
goto :TERMINATE_WITH_ERROR

:ERR_TARGET_DIR_NOT_SET
echo ERROR: Target dir MY_TARGET_DIR not set as environment variable!
goto :TERMINATE_WITH_ERROR

:ERR_TARGET_DIR_NOT_CREATABLE
echo ERROR: Target dir %targetDir% could not be created
goto :TERMINATE_WITH_ERROR

:TERMINATE_WITH_ERROR
echo.
echo ...%thisFile% terminated with error
goto :eof