////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of an automatic lock mechanism using an critical section
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once

///\cond

#include "windows.h" // CRITICAL_SECTION

namespace TestToolBox
{

template <class T>
class ObjectLockT
{
public:
	ObjectLockT(T* in_pObject)
	{
		if (in_pObject)
        {
			in_pObject->Lock();
        }
		m_pObject = (in_pObject);
	}

	~ObjectLockT()
	{
		if (m_pObject )
			m_pObject ->Unlock();
	}
	T* m_pObject;
};

class AutoCriticalSection
{
public:
	AutoCriticalSection() {InitializeCriticalSection(&m_sec);}
	~AutoCriticalSection() {DeleteCriticalSection(&m_sec);}
private:
	void Lock() {EnterCriticalSection(&m_sec);}
	void Unlock() {LeaveCriticalSection(&m_sec);}
	CRITICAL_SECTION m_sec;
    friend class AutoLock;
};

class AutoLock
{
public:
	AutoLock(AutoCriticalSection* in_pObject)
	{
		if (in_pObject)
        {
			in_pObject->Lock();
        }
		m_pObject = (in_pObject);
	}

	~AutoLock()
	{
		if (m_pObject )
			m_pObject ->Unlock();
	}
	AutoCriticalSection* m_pObject;
};

} // namespace TestToolBox

///\endcond
