////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of class BoostTestEventSinkForSynchronizer
///              Event message sink receiving events from Synchronizer and
///               generating appropriate Boost error events
///
/// \sa BoostEventReceiverForSynchronizer.h (File documentation)
///
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TestToolBox/IEventReceiver.h"
#include <boost/test/auto_unit_test.hpp>

///\cond do not document

namespace TestToolBox
{

	class BoostEventReceiverForSynchronizer : public IEventReceiver
	{
		// Signal actual event as occured during test execution
		virtual void EventMsg(
			EventContext in_context,
			const char*  in_eventDescription)
		{
            // Generate message containing two lines to enable both jumping to
            // this line and the line where the Sync-Macro was called
            // (assume that in_eventDescription contains location info)
            std::ostringstream fullInfoStr;
            if (in_context & CTX_TIMEOUT_ELAPSED) fullInfoStr << " SYNC: TIMEOUT_ELAPSED";
            if (in_context & CTX_TOO_MANY_EVENTS) fullInfoStr << " SYNC: TOO_MANY_EVENTS";
            if (in_context & CTX_FATAL_ERROR)     fullInfoStr << " SYNC: FATAL_ERROR";

            fullInfoStr << " (for more info see next line)\n" << in_eventDescription;

            if (in_context & CTX_FATAL_ERROR)
            {
                BOOST_FAIL(fullInfoStr.str());
            }
            else
            {
                BOOST_ERROR(fullInfoStr.str());
            }
        }
	};

}; // namespace TestToolBox

///\endcond do not document

