////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of class BoostTestEventSinkForTestEvents.
/// Event message sink receiving events from TestEvents
/// and generating appropriate Boost error events
///
/// \sa BoostEventReceiverForTestEvents.h (File documentation)
///
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TestToolBox/IEventReceiver.h"

///\cond do not document

namespace TestToolBox
{

	class BoostEventReceiverForTestEvents : public IEventReceiver
	{
		// Signal actual event as occured during test execution
		virtual void EventMsg(
			EventContext in_context,
			const char*  in_eventDescription)
		{
            // Generate message containing two lines to enable both jumping to
            // this line and the line where the TestEvent-Macro was called
            // (assume that in_eventDescription contains location info)
            std::ostringstream fullInfoStr;
            fullInfoStr << "TEST_EVENT:";
            if (in_context & CTX_INFO)
            {
                fullInfoStr << " INFO: " << in_eventDescription;
            }
            else
            {
                if (in_context & CTX_UNEXPECTED_EVENTS)
                {
                    fullInfoStr << " UNEXPECTED_EVENTS";
                }
                else
                {
                    fullInfoStr << " WRONG RESULT";
                }
                if (in_context & CTX_FATAL_ERROR)     fullInfoStr << " FATAL_ERROR";

                fullInfoStr << " (for more info see next line)\n"  << in_eventDescription;
            }

            if (in_context & CTX_FATAL_ERROR)
            {
                BOOST_FAIL(fullInfoStr.str());
            }
            else if (in_context & CTX_INFO)
            {
                BOOST_MESSAGE(fullInfoStr.str());
            }
            else
            {
                BOOST_ERROR(fullInfoStr.str());
            }
        }
	};

}; // namespace TestToolBox

///\endcond do not document

