////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// @{
/// \ingroup GrpProtocol 
/// \brief interface to write textual output to a protocol file
///
/// \sa IProtocol.h (File documentation)
/// \sa TestToolBox::IProtocol (interface documentation)
///
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <string>
#include "CommonSources/CommonDefinitions.h"

namespace TestToolBox
{

namespace OutputMode
{
const unsigned char M_NULL    = 0x00;  // no output
const unsigned char M_STDOUT  = 0x01;  // output to standard out
const unsigned char M_OUTFILE = 0x02;  // output to protocol file
const unsigned char M_RPTFILE = 0x04;  // output to report file
const unsigned char M_ALL = M_STDOUT | M_OUTFILE | M_RPTFILE;
};

    struct IProtocol;

    // ----------  Interface IProtocol  ----------

    /// \brief This interface is used for writing arbitrary text to a protocol file
    struct IProtocol
    {
        /// \brief Write arbitrary text (with or without newline chars)
        /// to protocol file.
        virtual void Write(
            std::string const & in_text) = 0;

        /// \brief Write arbitrary text (preferable without newline chars)
        /// to protocol file.
        virtual void Writeln(
            std::string const & in_text = "") = 0;

        /// \brief Write arbitrary text (preferable without newline chars)
        /// to protocol file.
        virtual void Writeln(
            OutputLevel::Enum   in_outputLevel,
            unsigned char       in_outputMode,
            std::string const & in_text = "") = 0;

        /// \brief Write footer info to test protocol.
        virtual void WriteFooter(unsigned char in_outputMode = TestToolBox::OutputMode::M_ALL) = 0;

        /// \brief Select output devices
        virtual void SetMode (
            unsigned char in_newMode) = 0;

        /// \brief Redirect output to all existing devices
        virtual void ResetMode (void) = 0;
    };


}; // namespace TestToolBox

