////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of class XmlCheck
///
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "XmlCheck.h"
#include "IEventReceiver.h"
#include "CommonSources\CommonDefinitions.h"

#include "TinyXPath\TinyXml.h"
#pragma warning(push)
#pragma warning(disable:4267)
#pragma warning(disable:4996)
#include "TinyXPath\xpath_static.h"
#pragma warning(pop) 

#include <iostream>
#include <iomanip>

///\cond do not document

// Switch to use as inline code (to be included in each cpp file)
// or as regular functions to be compiled within a lib or e.g. within stdafx.cpp
#ifdef TTB_NOT_INLINE
    #define TTB_INLINE
#else
    #define TTB_INLINE inline
#endif


//*****************************************************************************
//*****************************************************************************


namespace TestToolBox
{

    //************************************************************************

    namespace
    {
        TTB_INLINE std::string  WritePointerValue (std::string const & in_pointer)
        {
            if (in_pointer=="")
            {
                return "not set";
            }
            else if ((in_pointer=="0") || (in_pointer=="00000000"))
            {
                return "0";
            }
            else
            {
                return "exists";
            }
        }
    }

    //************************************************************************

TTB_INLINE XmlCheck::XmlCheck()
    : m_pEventReceiver   (0)
{
    TRCF("XmlCheck::XmlCheck");
    TRC(TL_DEBUG, "Constructor");

    SetDefaults();

} // XmlCheck::XmlCheck


//************************************************************************
//************************************************************************


TTB_INLINE XmlCheck::~XmlCheck()
{
    TRCF("XmlCheck::~XmlCheck");
    TRC(TL_DEBUG, "Destructor");

} // XmlCheck::~XmlCheck


//************************************************************************
//************************************************************************


TTB_INLINE XmlCheck* TheXmlCheck (void)
{
    return XmlCheck::Get();
}


TTB_INLINE XmlCheck* XmlCheck::Get (void)
{
    if (!s_pXmlCheck) s_pXmlCheck = new XmlCheck;
    return s_pXmlCheck;
}

TTB_INLINE void XmlCheck::Cleanup (void)
{
    if (s_pXmlCheck)
    {
        delete s_pXmlCheck;
        s_pXmlCheck = 0;
    }
}

//************************************************************************
//************************************************************************


TTB_INLINE IEventReceiver* XmlCheck::SetEventReceiver (
    IEventReceiver* in_pEventReceiver)
{
    IEventReceiver* retVal = m_pEventReceiver;
    m_pEventReceiver = in_pEventReceiver;
    return retVal;

} // XmlCheck::SetEventReceiver


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::SetDefaults (void)
{
    m_outputMode = CTX_TEST_EVENT;
    m_detailedOutput = false;
    m_attributesToIgnore.clear();
    m_nodesToIgnore.clear();
    m_doubleAttributes.clear();
    m_doubleNodes.clear();
    m_pointerAttributes.clear();
    m_pointerNodes.clear();
}


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::SetOutputModeForWritingXmlContents (
    EventContext in_outputMode)
{
    if (    (in_outputMode != CTX_PROT)
         && (in_outputMode != CTX_INFO)
         && (in_outputMode != CTX_TEST_EVENT))
    {
        std::ostringstream oss;
        oss << "Invalid output mode cannot be set, mode=" << in_outputMode;
        if (m_pEventReceiver)
            m_pEventReceiver->EventMsg(CTX_ERROR, oss.str().c_str());
    }
    else
    {
        m_outputMode = in_outputMode;
    }

} // XmlCheck::SetOutputModeForWritingXmlContents


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::SetDetailedOutput (
    bool in_detailedOutput)
{
    m_detailedOutput = in_detailedOutput;

} // XmlCheck::SetDetailedOutput


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::IgnoreAttribute (
        std::string const & in_attribute,
        bool                in_ignore)
{
    if (in_ignore)
    {
        m_attributesToIgnore.insert (in_attribute);
    }
    else
    {
        m_attributesToIgnore.erase (in_attribute);
    }

} // XmlCheck::IgnoreAttribute


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::IsAttributeToIgnore (
    std::string const & in_attribute) const
{
    return m_attributesToIgnore.find(in_attribute) != m_attributesToIgnore.end();

} // XmlCheck::IsAttributeToIgnore


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::IgnoreNode (
        std::string const & in_nodeName,
        bool                in_ignore)
{
    if (in_ignore)
    {
        m_nodesToIgnore.insert (in_nodeName);
    }
    else
    {
        m_nodesToIgnore.erase (in_nodeName);
    }

} // XmlCheck::IgnoreNode


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::IsNodeToIgnore (
    std::string const & in_nodeName) const
{
    return m_nodesToIgnore.find(in_nodeName) != m_nodesToIgnore.end();

} // XmlCheck::IsNodeToIgnore


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::UseAsDoubleAttribute (
    std::string const & in_attribute,
    bool                in_useAsDouble,
    int                 in_precision)
{
    if (in_useAsDouble)
    {
        m_doubleAttributes[in_attribute] = in_precision;
    }
    else
    {
        m_doubleAttributes.erase (in_attribute);
    }

} // XmlCheck::UseAsDoubleAttribute


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::IsDoubleAttribute (
        std::string const & in_attribute) const
{
    return m_doubleAttributes.find(in_attribute) != m_doubleAttributes.end();

} // XmlCheck::IsDoubleAttribute


//************************************************************************
//************************************************************************


TTB_INLINE int XmlCheck::GetPrecisionOfDoubleAttribute (
    std::string const & in_attribute) const
{
    int retVal = 0;
    std::map<std::string,int>::const_iterator it = m_doubleAttributes.find(in_attribute);
    if (it != m_doubleAttributes.end())
    {
        retVal = it->second;
    }
    return retVal;

} // XmlCheck::GetPrecisionOfDoubleAttribute 


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::UseAsDoubleNode (
    std::string const & in_nodeName,
    bool                in_useAsDouble,
    int                 in_precision)
{
    if (in_useAsDouble)
    {
        m_doubleNodes[in_nodeName] = in_precision;
    }
    else
    {
        m_doubleNodes.erase (in_nodeName);
    }

} // XmlCheck::UseAsDoubleNode


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::IsDoubleNode (
        std::string const & in_nodeName) const
{
    return m_doubleNodes.find(in_nodeName) != m_doubleNodes.end();

} // XmlCheck::IsDoubleNode


//************************************************************************
//************************************************************************


TTB_INLINE int XmlCheck::GetPrecisionOfDoubleNode (
    std::string const & in_nodeName) const
{
    int retVal = 0;
    std::map<std::string,int>::const_iterator it = m_doubleNodes.find(in_nodeName);
    if (it != m_doubleNodes.end())
    {
        retVal = it->second;
    }
    return retVal;

} // XmlCheck::GetPrecisionOfDoubleNode 


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::UseAsPointerAttribute (
    std::string const & in_attribute,
    bool                in_useAspointer)
{
    if (in_useAspointer)
    {
        m_pointerAttributes.insert (in_attribute);
    }
    else
    {
        m_pointerAttributes.erase (in_attribute);
    }

} // XmlCheck::UseAsPointerAttribute


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::IsPointerAttribute (
        std::string const & in_attribute) const
{
    return m_pointerAttributes.find(in_attribute) != m_pointerAttributes.end();

} // XmlCheck::IsPointerAttribute


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::UseAsPointerNode (
    std::string const & in_nodeName,
    bool                in_useAspointer)
{
    if (in_useAspointer)
    {
        m_pointerNodes.insert (in_nodeName);
    }
    else
    {
        m_pointerNodes.erase (in_nodeName);
    }

} // XmlCheck::UseAsPointerAttribute


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::IsPointerNode (
        std::string const & in_nodeName) const
{
    return m_pointerNodes.find(in_nodeName) != m_pointerNodes.end();

} // XmlCheck::IsPointerAttribute


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::CheckNode (
        TiXmlNode*   in_pNode,
        char const * in_file,
        long         in_lineNum)
{
    WriteNode (in_pNode, 0, 0, in_file, in_lineNum);

} // XmlCheck::CheckNode


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::CheckSubNode (
    TiXmlNode*          in_pNode,
    std::string const & in_xPathExpression_1,
    std::string const & in_xPathExpression_2,
    std::string const & in_xPathExpression_3,
    std::string const & in_xPathExpression_4,
    std::string const & in_xPathExpression_5,
    char const *        in_file,
    long                in_lineNum)
{
    WriteNode (
        SubNodeFromXPath(in_pNode, in_xPathExpression_1, in_xPathExpression_2,
            in_xPathExpression_3, in_xPathExpression_4, in_xPathExpression_5),
        0, 0, in_file, in_lineNum);

} // XmlCheck::CheckSubNode


//************************************************************************
//************************************************************************


TTB_INLINE void XmlCheck::CheckSubNodeWithMaxChilds (
    int                 in_maxChilds,
    TiXmlNode*          in_pNode,
    std::string const & in_xPathExpression_1,
    std::string const & in_xPathExpression_2,
    std::string const & in_xPathExpression_3,
    std::string const & in_xPathExpression_4,
    std::string const & in_xPathExpression_5,
    char const *        in_file,
    long                in_lineNum)
{
    WriteNode (
        SubNodeFromXPath(in_pNode, in_xPathExpression_1, in_xPathExpression_2,
            in_xPathExpression_3, in_xPathExpression_4, in_xPathExpression_5),
        0, in_maxChilds, in_file, in_lineNum);

} // XmlCheck::CheckSubNodeWithMaxChilds


//************************************************************************
//************************************************************************

/*
TTB_INLINE void TrimStr(std::string& io_str)
{
    std::string::size_type pos = io_str.find_last_not_of(" \r\n");
    if(pos != std::string::npos)
    {
        io_str.erase(pos + 1);
        pos = io_str.find_first_not_of(" \r\n");
        if(pos != std::string::npos)
        {
            io_str.erase(0, pos);
        }
    }
    else
    {
        io_str.erase(io_str.begin(), io_str.end());
    }
}
*/

TTB_INLINE bool XmlCheck::Compare (
        TiXmlNode*   in_pNodeA,
        TiXmlNode*   in_pNodeB,
        char const * in_file,
        long         in_lineNum)
{
    std::ostringstream oss;

    FormattedContents contA;
    WriteNode (in_pNodeA, 0, 0, in_file, in_lineNum, &contA);

    FormattedContents contB;
    WriteNode (in_pNodeB, 0, 0, in_file, in_lineNum, &contB);

    // Compare the two text containers with read XML data
    bool differenceFound = false;
    FormattedContents::iterator itA = contA.begin();
    FormattedContents::iterator itB = contB.begin();
    while (    (itA != contA.end())
            && (!differenceFound))
    {
        // Remove leading and trailing white space
        std::string strA = itA->m_text;
        TrimStr(strA);

        std::string strB;
        if (itB != contB.end())
        {
            strB = itB->m_text;
            TrimStr(strB);
        }

        // Compare the strings
        if (strA != strB)
        {
            differenceFound = true;
            oss << "Difference detected\n"
                << "A: " << strA
                << " (line: " << itA->m_row << " col: " << itA->m_col
                << ", path: " << itA->m_fullPath << ")"
                << std::endl;
            if (itB != contB.end())
            {
                oss << "B: " << strB
                    << " (line: " << itB->m_row << " col: " << itB->m_col
                << ", path: " << itB->m_fullPath << ")";
            }
            else
            {
                oss << "B: no data found";
            }
        }

        ++itA;
        if (itB != contB.end())
        {
            ++itB;
        }
    }

    // Check if B has additional data
    if (    (!differenceFound)
         && (itA == contA.end())
         && (itB != contB.end()))
    {
            std::string strB = itB->m_text;
            TrimStr(strB);

            oss << "Difference detected, B has more data\n"
                << "B: " << strB
                << " (line: " << itB->m_row << " col: " << itB->m_col << ")";
    }

    if (differenceFound)
    {
        m_compareResult = oss.str().c_str();
    }
    else
    {
        m_compareResult = "XML-Compare: no differences found";
    }

    if (m_pEventReceiver)
    {
        XmlCheck::StringPosition curPos;
        while (GetNextLineStr(m_compareResult, curPos))
        {
            m_pEventReceiver->Event(m_outputMode, curPos.m_lineStr);
        }

    }

    return !differenceFound;

} // XmlCheck::Compare


//************************************************************************
//************************************************************************


TTB_INLINE std::string XmlCheck::GetCompareResult (void) const
{
    return m_compareResult;

} // XmlCheck::GetCompareResult


//************************************************************************
// Helper functions
//************************************************************************


TTB_INLINE std::string XmlCheck::GetFullPath (
    TiXmlNode*          in_pNode,
    std::string const & in_rPathToConcat)
{
    std::string fullPath;

    if (in_rPathToConcat.empty())
    {
        fullPath = "/" + in_rPathToConcat;
    }

    TiXmlNode* pNode = in_pNode;
    do 
    {
        fullPath = std::string("/") + pNode->Value() + fullPath;
    }
    while ((pNode = pNode->Parent())!= 0);

    return fullPath;
    
} // XmlCheck::GetFullPath


//************************************************************************
//************************************************************************


TTB_INLINE TiXmlNode* XmlCheck::SubNodeFromXPath (
    TiXmlNode*          in_pParentNode,
    std::string const & in_xPathExpression_1,
    std::string const & in_xPathExpression_2,
    std::string const & in_xPathExpression_3,
    std::string const & in_xPathExpression_4,
    std::string const & in_xPathExpression_5)
{
    TiXmlNode * pNode = 0;
    pNode = TinyXPath::XNp_xpath_node (in_pParentNode, in_xPathExpression_1.c_str());
    if (pNode && !in_xPathExpression_2.empty())
    {
        pNode = TinyXPath::XNp_xpath_node (pNode, in_xPathExpression_2.c_str());
        if (pNode && !in_xPathExpression_3.empty())
        {
            pNode = TinyXPath::XNp_xpath_node (pNode, in_xPathExpression_3.c_str());
            if (pNode && !in_xPathExpression_4.empty())
            {
                pNode = TinyXPath::XNp_xpath_node (pNode, in_xPathExpression_4.c_str());
                if (pNode && !in_xPathExpression_5.empty())
                {
                    pNode = TinyXPath::XNp_xpath_node (pNode, in_xPathExpression_5.c_str());
                }
            }
        }
    }
    return pNode;

} // XmlCheck::SubNodeFromXPath


//************************************************************************
//************************************************************************


TTB_INLINE bool XmlCheck::GetNextLineStr (
    std::string const & in_stringWithNewLines,
    StringPosition &    io_current)
{
       if (io_current.m_lastPos == std::string::npos) return false;

       io_current.m_curPos = in_stringWithNewLines.find_first_of( '\n', io_current.m_lastPos);
       io_current.m_lineStr = in_stringWithNewLines.substr(io_current.m_lastPos,
           io_current.m_curPos - io_current.m_lastPos);

       if (io_current.m_curPos != std::string::npos)
           io_current.m_lastPos = io_current.m_curPos + 1;
       else
           io_current.m_lastPos = std::string::npos;

       return true;

} // XmlCheck::GetNextLineStr


//************************************************************************
// Private methods
//************************************************************************


TTB_INLINE void XmlCheck::WriteNode(
        TiXmlNode*         in_pNode,
        unsigned int       in_indent,
        int                in_maxChildNodes,
        char const *       in_file,
        long               in_lineNum,
        FormattedContents* in_pContents)
{
    TRCF("XmlCheck::WriteNode");
    TRC(TL_DEBUG, "Begin");

    if ( !in_pNode ) return;

    TiXmlNode* pChild;
    int nodeType = in_pNode->Type();

    std::ostringstream oss;
    oss << IndentStr(in_indent);

    bool writeAttributes = false;
    switch (nodeType)
    {
        case TiXmlNode::DOCUMENT:
            oss << IfDetailed("Document");
            break;

        case TiXmlNode::ELEMENT:
            oss << IfDetailed("Element: ")<< in_pNode->Value();
            writeAttributes = true;
            break;

        case TiXmlNode::COMMENT:
            oss << IfDetailed("Comment: ") << in_pNode->Value();
            break;

        case TiXmlNode::UNKNOWN:
            printf( "Unknown node" );
            break;

        case TiXmlNode::TEXT:
            // Check if parent element is of type double
            if (IsDoubleNode(in_pNode->Parent()->Value()))
            {
                std::istringstream isstr(in_pNode->ToText()->Value());
                double doubleVal;
                isstr>>doubleVal;
                {
                    oss << std::fixed << std::setprecision(GetPrecisionOfDoubleNode(in_pNode->Parent()->Value())) << doubleVal;
                }
            }
            else if (IsPointerNode(in_pNode->Parent()->Value()))
            {
                oss << WritePointerValue(std::string(in_pNode->ToText()->Value()));
            }
            else
            {
                oss << IfDetailed("Text: ") << in_pNode->ToText()->Value();
            }
            break;

        case TiXmlNode::DECLARATION:
            oss << IfDetailed("Declaration") << " version=" << in_pNode->ToDeclaration()->Version();
            break;

        default:
            oss << "Unexpected";
            break;
    }

    if (m_detailedOutput)
    {
        oss << " (line: " << in_pNode->Row() << ", col: " << in_pNode->Column()<< ")";
    }

    if (oss.str().length()>0)
    {
        if (in_pContents)
        {
            in_pContents->push_back(TextWithPosInfo(
                in_pNode->Row(),in_pNode->Column(), GetFullPath(in_pNode),
                oss.str().c_str()));
        }
        else if (m_pEventReceiver)
        {
            m_pEventReceiver->EventMsg(m_outputMode, oss.str().c_str());
        }
    }

    if (writeAttributes)
    {
        WriteAttributes(in_pNode->ToElement(), in_indent+1, in_file, in_lineNum, in_pContents);
    }

    // Iterate over children
    int numReportedChilds=0;
    for ( pChild = in_pNode->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
    {
        if (IsNodeToIgnore(pChild->Value())) continue;
        WriteNode( pChild, nodeType != TiXmlNode::DOCUMENT ? in_indent+1 : in_indent, 0, in_file, in_lineNum, in_pContents );
        ++numReportedChilds;
        if (in_maxChildNodes && (numReportedChilds >= in_maxChildNodes))
            break;
    }

    TRC(TL_DEBUG, "End");

} // XmlCheck::WriteNode


//************************************************************************
//************************************************************************

TTB_INLINE void XmlCheck::WriteAttributes(
        TiXmlElement*      in_pElement,
        unsigned int       in_indent,
        char const *       in_file,
        long               in_lineNum,
        FormattedContents* in_pContents)
{
    if ( !in_pElement ) return;

    TiXmlAttribute* pAttrib=in_pElement->FirstAttribute();
    int    numAttributes=0;
    int    intVal;
    double doubleVal;


    while (pAttrib)
    {
        if (!IsAttributeToIgnore(pAttrib->Name()))
        {
            std::ostringstream oss;
            oss << IndentStr(in_indent)
                << pAttrib->Name() << ": ";
            if (IsDoubleAttribute(pAttrib->Name()))
            {
                if (pAttrib->QueryDoubleValue(&doubleVal)==TIXML_SUCCESS)
                {
                    oss << std::fixed << std::setprecision(GetPrecisionOfDoubleAttribute(pAttrib->Name())) << doubleVal;
                }

            }
            else if (IsPointerAttribute(pAttrib->Name()))
            {
                oss << WritePointerValue(std::string(pAttrib->Value()));
            }
            else
            {
                oss << pAttrib->Value();
                if (m_detailedOutput)
                {
                    // Try to read as integer
                    if (pAttrib->QueryIntValue(&intVal)==TIXML_SUCCESS)
                    {
                        oss << ", as int: " << intVal;
                    }

                    // Try to read as double
                    if (pAttrib->QueryDoubleValue(&doubleVal)==TIXML_SUCCESS)
                    {
                        oss << ", as double: " << doubleVal;
                    }
                }
            }

            if (m_detailedOutput)
            {
                oss << " (line: " << pAttrib->Row() << ", col: " << pAttrib->Column()<< ")";
            }

            if (in_pContents)
            {
                in_pContents->push_back(TextWithPosInfo(
                    pAttrib->Row(),pAttrib->Column(), GetFullPath(in_pElement, pAttrib->Name()),
                    oss.str().c_str()));
            }
            else if (m_pEventReceiver)
            {
                m_pEventReceiver->EventMsg(m_outputMode, oss.str().c_str());
            }

            numAttributes++;
        }
        pAttrib=pAttrib->Next();
    }

    if (m_detailedOutput && (numAttributes > 0))
    {
        std::ostringstream oss;
        oss << IndentEmptyStr(in_indent) << numAttributes << " attributes";
        if (in_pContents)
        {
            in_pContents->push_back(TextWithPosInfo(
                pAttrib->Row(),pAttrib->Column(), GetFullPath(in_pElement, pAttrib->Name()),
                oss.str().c_str()));
        }
        else if (m_pEventReceiver)
        {
            m_pEventReceiver->EventMsg(m_outputMode, oss.str().c_str());
        }
    }

} // XmlCheck::WriteAttributes


//************************************************************************
//************************************************************************


TTB_INLINE std::string XmlCheck::IfDetailed (std::string const & in_str)
{
    return m_detailedOutput ? in_str : "";
}

//************************************************************************
//************************************************************************

///\endcond

} // namespace TestToolBox


