////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of ShortNames
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "./CommonSources/CommonDefinitions.h"
#include "ShortNames.h"
#include "SimulatedObjectBase.h"
#include "TestEvents.h"

namespace TestToolBox
{
    namespace ShortNames
    {
        TTB_INLINE RegisteredSimulatedObject* SimObject(std::string const & in_objectName)
        {
            RegisteredSimulatedObject* pObject = TheRegisteredSimulatedObjects()->
                Get(in_objectName);
            if (!pObject)
            {
                TTB_ACT_S("SimObject: ERROR: Could not find simulated object with name: " << in_objectName);
                TTB_ACT_S("SimObject: The following objects are currently registered:");
                TheRegisteredSimulatedObjects()->WriteObjectsToTestEvents();
            }
            return pObject;
        }

#pragma warning( push )
#pragma warning( disable : 4996 ) // vsprintf declared deprecated

        TTB_INLINE void Act(const char* in_actualEvent, ...)
        {
            UTIL_GET_FORMATTED_LINE_NO_NEWLINE(in_actualEvent);
            TestEvents::Get()->Act(formattedLine);

        } // Act

#pragma warning( pop )
        //************************************************************************
        //************************************************************************


        TTB_INLINE void ReportStoredErrors(void)
        {
            MyErrorEvents::Get()->ReportStoredErrors();

        } // ReportStoredErrors

        //************************************************************************

        TTB_INLINE void DeleteStoredErrors(void)
        {
            MyErrorEvents::Get()->DeleteStoredErrors();

        } // DeleteStoredErrors

        //************************************************************************

        TTB_INLINE void SetIgnoreError(
            long in_errorMsgNum,
            bool in_ignore)
        {
            MyErrorEvents::Get()->SetNotificationActionForErrorMsgNum(
                in_errorMsgNum, in_ignore ? ACTION_IGNORE : ACTION_NOT_SPECIFIED);

        } // ReportStoredErrors

        //************************************************************************

        TTB_INLINE void IgnoreErrorContainingTextPattern(
            std::string const & in_text)
        {
            MyErrorEvents::Get()->IgnoreErrorContainingTextPattern(in_text);

        } // IgnoreErrorContainingTextPattern

        //************************************************************************

        TTB_INLINE void ClearStoredErrorTextPatterns(void)
        {
            MyErrorEvents::Get()->ClearStoredTextPatterns();

        } // ClearStoredErrorTextPatterns

        //************************************************************************

        TTB_INLINE void SetErrorSync(
            bool in_syncOnError)
        {
            MyErrorEvents::Get()->SetErrorSync(in_syncOnError);

        } // SetErrorSync

        //************************************************************************

        TTB_INLINE void SetImmediateErrorLog(
            bool in_immediateLog)
        {
            MyErrorEvents::Get()->SetImmediateErrorLog(in_immediateLog);

        } // SetImmediateErrorLog


        //************************************************************************

        TTB_INLINE bool ErrorOk(
            long in_extErrorId)
        {
            return MyErrorEvents::Get()->ErrorOk(in_extErrorId);

        } // SetImmediateErrorLog
    }
}
