////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of TestInstanceDll
/// 
////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"

#include "TestToolBox\EnvironmentImpl.h"
#include "TestToolBox\ProtocolImpl.h"
#include "TestToolBox\SynchronizerImpl.h"
#include "TestToolBox\TestEventsImpl.h"
#include "TestToolBox\TestStreamImpl.h"
#include "TestToolBox\TestTimeImpl.h"
#include "TestToolBox\SimulatedObjectBaseImpl.h"

///\cond do not document

// Define name of this component for simple trace utility and set initial trace level
TTB_TRC_DEF_GLOBALS ("TestInstanceDll", TL_VAL_DEBUG);

#ifdef _MANAGED
#pragma managed(push, off)
#endif


BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}


//-----------------------------------------------------------------------------

static TestToolBox::Environment*                 fglobal_pEnvironment = 0;
static TestToolBox::Protocol*                    fglobal_pProtocol = 0;
static TestToolBox::Synchronizer*                fglobal_pSynchronizer = 0;
static TestToolBox::TestEvents*                  fglobal_pTestEvents = 0;
static TestToolBox::TestTime*                    fglobal_pTestTime = 0;
static TestToolBox::GlobalOptions*               fglobal_pGlobalOptions = 0;
static TestToolBox::RegisteredSimulatedObjects*  fglobal_pRegisteredSimulatedObjects = 0;
static TestToolBox::Formatter*                   fglobal_pFormatter = 0;

//-----------------------------------------------------------------------------

TestToolBox::Environment* TTB_GetInstance_Environment (void)
{
    if (!fglobal_pEnvironment)
    {
        fglobal_pEnvironment = new TestToolBox::Environment;
    }
    return fglobal_pEnvironment;
}

void TTB_CleanupInstance_Environment (void)
{
    delete fglobal_pEnvironment; fglobal_pEnvironment= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::Protocol * TTB_GetInstance_Protocol (
    unsigned int in_allowedStreams,
    TestToolBox::OutputLevel::Enum in_outputLevel)
{
    if (!fglobal_pProtocol)
    {
        fglobal_pProtocol = TestToolBox::Protocol::CreateNewInstanceForUsageInDll(in_allowedStreams, in_outputLevel);
    }
    return fglobal_pProtocol;
}

void TTB_CleanupInstance_Protocol (void)
{
    delete fglobal_pProtocol;
    fglobal_pProtocol= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::Synchronizer * TTB_GetInstance_Synchronizer (void)
{
    if (!fglobal_pSynchronizer)
    {
        fglobal_pSynchronizer = TestToolBox::Synchronizer::CreateNewInstanceForUsageInDll();
    }
    return fglobal_pSynchronizer;
}

void TTB_CleanupInstance_Synchronizer (void)
{
    delete fglobal_pSynchronizer;
    fglobal_pSynchronizer= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::TestEvents* TTB_GetInstance_TestEvents (void)
{
    if (!fglobal_pTestEvents)
    {
        fglobal_pTestEvents = TestToolBox::TestEvents::CreateNewInstanceForUsageInDll();
    }
    return fglobal_pTestEvents;
}

void TTB_CleanupInstance_TestEvents (void)
{
    delete fglobal_pTestEvents;
    fglobal_pTestEvents= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::TestTime* TTB_GetInstance_TestTime (void)
{
    if (!fglobal_pTestTime)
    {
        fglobal_pTestTime = TestToolBox::TestTime::CreateNewInstanceForUsageInDll();
    }
    return fglobal_pTestTime;
}

void TTB_CleanupInstance_TestTime (void)
{
    delete fglobal_pTestTime;
    fglobal_pTestTime= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::GlobalOptions* TTB_GetInstance_GlobalOptions (void)
{
    if (!fglobal_pGlobalOptions)
    {
        fglobal_pGlobalOptions = TestToolBox::GlobalOptions::CreateNewInstanceForUsageInDll();
    }
    return fglobal_pGlobalOptions;
}

void TTB_CleanupInstance_GlobalOptions (void)
{
    delete fglobal_pGlobalOptions;
    fglobal_pGlobalOptions= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::RegisteredSimulatedObjects* TTB_GetInstance_RegisteredSimulatedObjects (void)
{
    if (!fglobal_pRegisteredSimulatedObjects)
    {
        fglobal_pRegisteredSimulatedObjects = TestToolBox::RegisteredSimulatedObjects::CreateNewInstanceForUsageInDll();
    }
    return fglobal_pRegisteredSimulatedObjects;
}

void TTB_CleanupInstance_RegisteredSimulatedObjects (void)
{
    delete fglobal_pRegisteredSimulatedObjects;
    fglobal_pRegisteredSimulatedObjects= 0;
}

//-----------------------------------------------------------------------------

TestToolBox::Formatter* TTB_GetInstance_Formatter (void)
{
    if (!fglobal_pFormatter)
    {
        fglobal_pFormatter = TestToolBox::Formatter::CreateNewInstanceForUsageInDll();
    }
    return fglobal_pFormatter;
}

void TTB_CleanupInstance_Formatter (void)
{
    delete fglobal_pFormatter;
    fglobal_pFormatter= 0;
}

//-----------------------------------------------------------------------------

#ifdef _MANAGED
#pragma managed(pop)
#endif

///\endcond
