////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of class SimpleObject
/// 
////////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include "SimpleObject.h"
#include <boost/bind.hpp>
#include "TestToolBox/CommonSources/CommonDefinitions.h"

///\cond do not document


//************************************************************************
//
// Interface ISimpleObject
//
//************************************************************************


void SimpleObject::Connect(
    ICallBackFromSimpleObject* in_pICallBack)
{
    // Store callback interface to client
	m_pICallBack = in_pICallBack;

} // SimpleObject::Connect


//************************************************************************
//************************************************************************


void SimpleObject::CalcSync(long in_val)
{
    TRCF("CalcSync");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    if (m_pICallBack)
    {
        m_pICallBack-> Result (InternalCalc (in_val));
    }

    TRC(TL_DEBUG,"End");

} // SimpleObject::CalcSync


//************************************************************************
//************************************************************************


void SimpleObject::CalcAsync (long in_val)
{
    TRCF("CalcAsync");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    // Execute asynchronously within separate thread
    m_threads.create_thread(
        boost::bind(&SimpleObject::ExecCalcAsync,this, in_val));

    TRC(TL_DEBUG,"End");

} // SimpleObject::CalcAsync


//************************************************************************
// Other member functions
//************************************************************************


void SimpleObject::ExecCalcAsync (long in_val)
{

    TRCF("ExecCalcAsync");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    if (m_pICallBack)
    {
        m_pICallBack-> Result (InternalCalc (in_val));
    }

    TRC(TL_DEBUG,"End");

} // SimpleObject::ExecCalcAsync


//************************************************************************
//************************************************************************

///\endcond do not document
