////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Class SimpleObject with synchronous, asynchronous and blocking interface
/// methods which are used to demonstrate the usage of
/// test utilities.
/// 
////////////////////////////////////////////////////////////////////////////////


#pragma once

///\cond do not document

#include "ISimpleObject.h"
#include <boost/thread.hpp>


class SimpleObject : public ISimpleObject
{
public:

    // Construction
	SimpleObject()
        :
        m_pICallBack     (0)
	{}

    // Destruction
    ~SimpleObject()
    {m_threads.join_all();};


    // Interface ISimpleObject
	void Connect(ICallBackFromSimpleObject* in_pICallBack);
	void CalcSync(long in_val);
	void CalcAsync(long in_val);

private:

    // Callback interface to client
    ICallBackFromSimpleObject* m_pICallBack;

    // Threads used for asynchronous execution
    boost::thread_group m_threads;

private:

    // Perform simple numeric calculation
    long InternalCalc (long in_val)
    {return in_val * 2;}

    // Routine called within a separate thread
	void ExecCalcAsync(long in_val);
};

///\endcond
