// ATTENTION: methods within this file are partially outdated.
// Consider using TTB_BOOST_TEST_CASE for simpler usage.

////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \defgroup GrpTestExtensionsForBoostTest TestExtensionsForBoost
/// @{
/// \ingroup GrpTestApps 
/// \brief
/// Win32 console application "TestExtensionsForBoostTest.exe" demonstrates
/// the running of a unit test with Boost.Test lib and some extension features.
/// 
/// \n The features contain:
///
///  - synchronisation of test execution running within
///    several threads
/// 
///  - automatic verification of asynchronously occured events
///    which may be in fixed or non deterministic order
///
///  - storage of application errors, verification as events,
///    possibility for filtering and displaying error trees.
///    <br> Remark: to connect with a specific ErrorHandling mechanism
///    a (small) adapter has to be added which translates the
///    project specific error format into the internal format which
///    can be handled
///
/// <h2>Example: Run without specific command line arguments</h2>
///
/// - General information and test cases are written to std out
/// - Detailed test output will be written to test protocol.
/// - Remark: command line param -suppressErrorTestCases is used to execute only those
///   test cases which will not detect a testing error
/// 
/// Console output:
/// \code
///
/// CmdLineOptions : -suppressErrorTestCases
/// 
/// Generic test protocol template / TestToolBox (TTB) 2010
/// 
/// 
///                     T E S T   P R O T O C O L
/// 
/// Date              : 10.01.2010 13:29:59
/// Test application  : C:\My Target\debug\TestExtensionsForBoostTest.exe
/// Test protocol     : C:\My Target\debug\TestExtensionsForBoostTest.out
/// Test report       : C:\My Target\debug\TestExtensionsForBoostTest.rpt
/// Source path       : c:\userdata\gerald\sw\c\testtoolbox
/// Command line args : -suppressErrorTestCases
/// Run on computer   : SIRIUS
/// Run by            : Gerald
/// ________________________________________________________________________________
/// Running 8 test cases...
/// Test: TestCase_WaitForSingleAsyncAnswer
/// Test: TestCase_WaitForMultipleAsyncAnswers
/// Test: TestCase_VariableSequenceOfReactions
/// Test: BaseTestCase_Events_FixedSequence
/// Test: BaseTestCase_Events_VariableSequence
/// Test: BaseTestCase_ErrorEvents_Generate
/// Test: BaseTestCase_ErrorEvents_Chains
/// Test: BaseTestCase_ErrorEvents_Configuration
/// ________________________________________________________________________________
/// 
/// Test started    : 10.01.2010 13:29:59
/// Test ended      : 10.01.2010 13:29:59
/// Duration        :            00:00:00,016
/// 
/// Tests performed : 8
/// Tests failed    : 0
/// 
/// No error detected!
/// 
/// *** No errors detected
/// \endcode
///
/// Tets protocol (TestExtensionsForBoost.out):
/// \code
/// ...
/// ================================================================================
/// Test: TestCase_WaitForSingleAsyncAnswer
/// ================================================================================
/// 
/// Calling CalcAsync(7)
/// (Waiting for asynchronous result via callback)
/// 
/// >CallBack-Result: 14
/// 
/// ================================================================================
/// Test: TestCase_WaitForMultipleAsyncAnswers
/// ================================================================================
/// 
/// 
/// <so.CalcSync          2
/// >CallBack-Result: 4
/// 
/// <so.CalcSync          33
/// >CallBack-Result: 66
/// ...
/// \endcode
///
/// <h2>Example: Minimum console output, no protocol files</h2>
///
/// - Command line params: <b>-noProtocol</b>
/// - Protocol files *.out and *.rpt are not created
/// - in case of error free test execution no special output (except messages from BOOST)
///   are written to stdout
/// 
/// Console output:
/// \code
/// CmdLineOptions : -noProtocol -suppressErrorTestCases
/// 
/// Running 8 test cases...
/// 
/// *** No errors detected
/// \endcode
///
/// <h2>Example: More output from BOOST, no protocol files</h2>
///
/// - Command line params: <b>-noProtocol -detailedLog 0 --log_level=test_suite</b>
/// - Protocol files *.out and *.rpt are not created
/// - in case of error free test execution no special output (except 
///   more detailed messages from BOOST) are written to stdout
/// 
/// Console output:
/// \code
/// CmdLineOptions : -noProtocol -detailedLog 0 --log_level=test_suite -suppressErrorTestCases
/// 
/// Running 8 test cases...
/// Entering test suite "Master Test Suite"
/// Entering test suite "All my test suites"
/// Entering test suite "Testing asynchronous reactions"
/// Entering test case "TestCase_WaitForSingleAsyncAnswer"
/// Leaving test case "TestCase_WaitForSingleAsyncAnswer"
/// Entering test case "TestCase_WaitForMultipleAsyncAnswers"
/// Leaving test case "TestCase_WaitForMultipleAsyncAnswers"
/// Entering test case "TestCase_VariableSequenceOfReactions"
/// Leaving test case "TestCase_VariableSequenceOfReactions"
/// Entering test case "BaseTestCase_Events_FixedSequence"
/// Leaving test case "BaseTestCase_Events_FixedSequence"
/// Entering test case "BaseTestCase_Events_VariableSequence"
/// Leaving test case "BaseTestCase_Events_VariableSequence"
/// Leaving test suite "Testing asynchronous reactions"
/// Entering test suite "Testing error chains"
/// Entering test case "BaseTestCase_ErrorEvents_Generate"
/// Leaving test case "BaseTestCase_ErrorEvents_Generate"
/// Entering test case "BaseTestCase_ErrorEvents_Chains"
/// Leaving test case "BaseTestCase_ErrorEvents_Chains"
/// Entering test case "BaseTestCase_ErrorEvents_Configuration"
/// Leaving test case "BaseTestCase_ErrorEvents_Configuration"
/// Leaving test suite "Testing error chains"
/// Leaving test suite "All my test suites"
/// Leaving test suite "Master Test Suite"
/// 
/// Running 8 test cases...
/// 
/// *** No errors detected
/// \endcode
///
/// <h2>Example: More output from BOOST and TestToolBox, no protocol files</h2>
///
/// - Command line params: <b>-noProtocol -detailedLog 1 --log_level=test_suite</b>
/// - Protocol files *.out and *.rpt are not created
///
/// Console output:
/// \code
/// CmdLineOptions : -noProtocol -detailedLog 1 --log_level=test_suite -suppressErrorTestCases
/// 
/// Running 8 test cases...
/// Entering test suite "Master Test Suite"
/// Entering test suite "All my test suites"
/// Entering test suite "Testing asynchronous reactions"
/// Entering test case "TestCase_WaitForSingleAsyncAnswer"
/// TEST_EVENT: INFO: Calling CalcAsync(7)
/// (Waiting for asynchronous result via callback)
/// TEST_EVENT: INFO: Act: CallBack-Result: 14
/// Leaving test case "TestCase_WaitForSingleAsyncAnswer"
/// Entering test case "TestCase_WaitForMultipleAsyncAnswers"
/// TEST_EVENT: INFO: Cmd: so.CalcSync 2
/// TEST_EVENT: INFO: Act: CallBack-Result: 4
/// TEST_EVENT: INFO: Cmd: so.CalcSync 33
/// TEST_EVENT: INFO: Act: CallBack-Result: 66
/// TEST_EVENT: INFO: Cmd: so.CalcSync 99
/// TEST_EVENT: INFO: Act: CallBack-Result: 198
/// Leaving test case "TestCase_WaitForMultipleAsyncAnswers"
/// ...
/// Leaving test suite "Testing error chains"
/// Leaving test suite "All my test suites"
/// Leaving test suite "Master Test Suite"
/// 
/// *** No errors detected
/// \endcode
///
/// <h2>Hint</h2>
/// You can try other combinations of command line params to receive the output as needed.
///
/// \sa TestExtensionsForBoostTest.cpp (file documentation)
/// \sa GrpEnvironmentForBoostTest (usage documentation)
///
////////////////////////////////////////////////////////////////////////////////

///\cond do not document

#include "stdafx.h"

#include <iostream>
using namespace std;

#include <boost/bind.hpp>

// Switch to activate/deactivate automatic registration of test cases
// If deactivated manual registration is used (i.e. function
// init_unit_test_suite() is used to setup the test sequence
// #define AUTO_REGISTER // this is a private define local within this file!

#include <boost/test/unit_test.hpp>


//*** Use existing boost test framework implementation ***

// Possibility 1:
// using test framework by explicitly linking with boost framework library
// Important: use the library version which is built with the same C/C++ runtime
// as the test executable is built with
//   mt : multithreaded
//    s : static runtime
//    d : debug build
//    g : debug/diagnostic runtime
//#pragma comment(lib, "boost_unit_test_framework-vc80-mt-gd-1_36")

// Possibility 2:
// Use the boost auto link feature which automatically chooses the correct library version
#define BOOST_LIB_NAME boost_unit_test_framework
#include <boost/config/auto_link.hpp>

// Possibility 3:
// instead of linking with the unit test library you can compile all required sources
// through the following include;
// disadvantage: longer compilation time
// #include <boost/test/included/unit_test_framework.hpp>


#ifdef AUTO_REGISTER
    #define BOOST_AUTO_TEST_MAIN
    #include <boost/test/auto_unit_test.hpp>
#endif

using boost::unit_test::test_suite;

// Use extensions for BoostTest
#include "TestToolBox\ExtensionsForBoostTest/BoostDefaultTestEnvironmentImpl.h"
#include "TestToolBox\EnvironmentImpl.h"
#include "TestToolBox\ProtocolImpl.h"
#include "TestToolBox\TestTimeImpl.h"
#include "TestToolBox\TestStreamImpl.h"
#include "TestToolBox\TestToolBoxImpl.h"


// Object to test
#include "SourcesToTest/SimpleObject.h"

// Callback object to receive calls from object to test
#include "CallBackFromSimpleObject.h"

using namespace TestToolBox;
namespace TTB = TestToolBox;

// Define name of this component for simple trace utility and set initial trace level
TTB_TRC_DEF_GLOBALS ("TestExtBoost", TL_VAL_DEBUG);


//*******************************************************************
// Workaround memory leak in boost thread onetime initialization code
// (see google search: boost memory leak thread, may be solved in new
// boost versions)
//*******************************************************************

// static WorkAroundBoostThreadMemLeak s;


//*******************************************************************
// Global test settings and cleanup
//*******************************************************************

// Cleanup of test protocol is automatically performed within destructor at end of all test cases
// We are using the same instances of TTB::Protocol and TTB::TestEvent within
// all Boost test cases (by specifying TTB test cases via TTB_AUTO_TEST_CASE and TTB_TEST_CASE).
// Therefore we have to do the cleanup at end of all tests by deriving from TTB::FinalCleanup
// or TTB::DefaultSettingsForBoostAndTTB.
// Hint: In stead of using TTB::DefaultSettingsForBoostAndTTB you can
// build your own settings class and construct the test environment
// from elements of TestToolBox as you need.
struct MyGlobalSettingsForTestToolBox : public TTB::DefaultSettingsForBoostAndTTB
{
    MyGlobalSettingsForTestToolBox (void)
    : TTB::DefaultSettingsForBoostAndTTB(
          TTB::TheEnvironment()->IsExistingCommandLineOption("-noProtocol")
              ? TTB::OutputMode::M_NULL : TTB::OutputMode::M_ALL)
    {
        // Per default the detailed logging of test events is activated and 
        // all calls of Act() are written to test protocol in format:
        // >Some event generated

        // Optionally you can switch off detailed protocol via command line param
        // -detailedLog 0 (syntax is specifically defined for this test application!)

        // Optionally you can specify a syntax for all generated test events which
        // allows direct copying from protocol to source code. (recommended when
        // creating/correcting test scripts!)
        // Via command line param '-prefixForAct TTB_EXP' you get the following format:
        // TTB_EXP("Some event generated");
        TTB::TestEvents::Get()->SetDetailedLog(
             // detailed log if not specified
             !TTB::TheEnvironment()->IsExistingCommandLineOption("-detailedLog")
             // or use settings from param
             || TTB::TheEnvironment()->GetCommandLineOptionVal<bool>("-detailedLog"),
            TTB::TheEnvironment()->GetCommandLineOption("-prefixEvents"));
    }
};
BOOST_GLOBAL_FIXTURE(MyGlobalSettingsForTestToolBox);


//*******************************************************************
// The test cases
//*******************************************************************


// Define convenience macro to both support auto registration and
// manual registration of test cases
// (switch by activating/deactivating macro AUTO_REGISTER
#ifdef AUTO_REGISTER
    #define TEST_CASE(name) BOOST_AUTO_TEST_CASE (name)
#else
    #define TEST_CASE(name) void name()
#endif


TEST_CASE(TestCase_WaitForSingleAsyncAnswer)
{
	// Setup TestEnvironment for test case,
    // Cleanup is automatically performed within destructor
    TTB_AUTO_TEST_CASE ("TestCase_WaitForSingleAsyncAnswer");

    // Instantiate the object to test
    SimpleObject so;

    // Instantiate a callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    // Prepare to receive a single sync event
  	TTB_INIT_SYNC(1);

    // Optionally you can describe your test case or the actions within it);
    TTB_INFO("Calling CalcAsync(7)\n"
             "(Waiting for asynchronous result via callback)");

    // Start a calculation
    // The result is asynchronpusly returned via callback interface
    // ICallBackFromSimpleObject::Result.
    so.CalcAsync(7);

    // Wait until the calculation has asynchronously returned the result
    TTB_WAIT_SYNC();

    // When the sync event has occured the result should be available
    // It was stored as test event (see call of TestEvents::Get()->Act
    // within CallBackFromSimpleObject::Result)
    TTB_EXP("CallBack-Result: 14");

} // TestCase_WaitForSingleAsyncAnswer


//*******************************************************************


TEST_CASE(TestCase_WaitForMultipleAsyncAnswers)
{
	// Setup TestEnvironment for test case,
    // Cleanup is automatically performed within destructor
    TTB_AUTO_TEST_CASE ("TestCase_WaitForMultipleAsyncAnswers");

    // Instantiate the object to test
    SimpleObject so;

    // Instantiate a callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);
	Sleep(5000);
    // Prepare to receive 3 sync events
  	TTB_INIT_SYNC(3);

    // Start 3 calculations which will return their results
    // asynchronously but in the same sequence

    // Optionally you can use macro TTB_CMD to call an method and also log
    // the call within test stream.
    // I.e. instead of writing
    // so.CalcSync(2);
    // so.CalcSync(33);
    // so.CalcSync(98);
    // both call and log using the following syntax:
    TTB_CMD1(so.CalcSync,2);
    TTB_CMD1(so.CalcSync,33);
    TTB_CMD1(so.CalcSync,99);

    // Remarks: To keep the code of the object under test simple
    // here the synchronous execution methods are called
    // otherwise the preserving of sequence would not be guaranteed.
    // In a real world application think of a situation where you make one
    // or more interactions and then you expect three reactions in exactly
    // the specified order

    // Wait until the calculation has asynchronously returned all results
    TTB_WAIT_SYNC();

    // When the sync events have occured all results should be available
    // Remark: The sequence of macro calls has to be in the same order as the corresponding
    // calls to CalcSync otherwise an error is reported.
    TTB_EXP ("CallBack-Result: 4");
    TTB_EXP1("CallBack-Result: %d",         66);  // example of using printf syntax
    TTB_EXP2("CallBack-%s: %d" , "Result", 198);  // example of using printf syntax

} // TestCase_WaitForMultipleAsyncAnswers


//*******************************************************************


TEST_CASE(TestCase_VariableSequenceOfReactions)
{
	// Setup TestEnvironment for test case,
    // Cleanup is automatically performed within destructor
    TTB_AUTO_TEST_CASE("TestCase_VariableSequenceOfReactions");

    // To demonstrate debugging possibilities:
    // Activate logging of test events, i.e. each time someone calls
    // TestEvent::Act/ActualResult the given text is written to testoutput.
    // As we are running within the Boost test framework this means that
    // an informational message is generated via BOOST_MESSAGE macro.
    // To watch this outpout the loglevel has to be changed via
    // commandline (--log_level=test_suite)
    // TestEvents::Get()->LogCallsToAct(true);

    // Instantiate the object to test
    SimpleObject so;

    // Instantiate an callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    // Now execute some calculations.
    // The answers may come in any order.

  	TTB_INIT_SYNC(4);

    // Start 4 calculations
    so.CalcAsync(1);
    so.CalcAsync(2);
    so.CalcAsync(3);
    so.CalcAsync(4);

    TTB_WAIT_SYNC();

    // When the sync events have occured all results should be available
    // Remark: The following macro calls indicate that the sequence of
    // events does not matter
    TTB_EXP_VAR ("CallBack-Result: 6");
    TTB_EXP_VAR ("CallBack-Result: 4");
    TTB_EXP_VAR ("CallBack-Result: 2");
    TTB_EXP_VAR ("CallBack-Result: 8");

} // TestCase_VariableSequenceOfReactions


//*******************************************************************


TEST_CASE(TestCase_DetectMultipleErrors)
{
	// Setup TestEnvironment for test case,
    // Cleanup is automatically performed within destructor
    TTB_AUTO_TEST_CASE ("TestCase_DetectMultipleErrors");

    // Instantiate the object to test
    SimpleObject so;
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    //-------------------------------------------
    TTB_SECTION("Sub test case 1");
    //-------------------------------------------

    // In the first sequence of actions two errors will be returned
    // (expecting wrong values 44 and 88). The error detection mechanism
    // of class TestEvents stops searching for errors when the first error
    // within a test case is found.
    // Motivation: Especially in multithreaded environments a single
    // error will often lead to many subsequent errors. By reporting
    // only the first error the test result is more readable.
    // Another obvious reason: The implementation of the error detection
    // algorithm is much more simple when stopping at the first error.

  	TTB_INIT_SYNC(4);

    // Start 4 calculations
    so.CalcAsync(1);
    so.CalcAsync(2);
    so.CalcAsync(3);
    so.CalcAsync(4);

    TTB_WAIT_SYNC();

    TTB_EXP_VAR ("CallBack-Result: 2");
    TTB_EXP_VAR ("CallBack-Result: 44");
    TTB_EXP_VAR ("CallBack-Result: 6");
    TTB_EXP_VAR ("CallBack-Result: 88");


    //-------------------------------------------
    TTB_SECTION("Sub test case 2");
    //-------------------------------------------

    // In the second sequence of actions a single error (wrong expected
    // result 4711) will be detected only if the following line of code
    // is activated:
    // TTB_TEST_CASE ("TestCase_DetectMultipleErrors (Sub test case 2)");

    // Remarks: With this call you can clear all events stored from former
    // test actions. In that way you reenable the compare mechanism which
    // has been deactivated when the first error is found in sub test case 1.

    // To demonstrate debugging possibilities:
    // List all stored events (this list will be generated automatically
    // when an error is detected within the EXP macros, see error detection
    // above in sub test case 1).
    // To watch this outpout the loglevel has to be changed via
    // commandline (--log_level=test_suite) or the TestEvents have to be connected to
    // TTB::Protocol
    TTB_LIST_ALL_STORED_EVENTS();

  	TTB_INIT_SYNC(2);

    // Start 2 calculations
    so.CalcAsync(7);
    so.CalcAsync(8);

    TTB_WAIT_SYNC();

    TTB_LIST_ALL_STORED_EVENTS();

    TTB_EXP_VAR ("CallBack-Result: 14");
    TTB_EXP_VAR ("CallBack-Result: 4711");

} // TestCase_DetectMultipleErrors


//*******************************************************************
//*******************************************************************


TEST_CASE(BaseTestCase_Sync_ReceivingMoreEventsThanExpected)
{
    TTB_AUTO_TEST_CASE ("BaseTestCase_Sync_ReceivingMoreEventsThanExpected");

    TTB_INIT_SYNC(1); // configure for receiving a single sync event

    // For simplicity the sync events are generated here
    // Real testing code will generate these events at some other location
    // probably running within one ore more separate threads
    TTB_SYNC();       // generate the first event
    TTB_SYNC();       // generate the second event, causes a test error
    TTB_SYNC();       // generate the third event, causes a test error

    TTB_WAIT_SYNC();  // wait returns immediately, there are 3 events but only 1
                     // expected which causes a test error

} // BaseTestCase_Sync_ReceivingMoreEventsThanExpected


//*******************************************************************


TEST_CASE(BaseTestCase_Sync_TimeoutReceivingLessEventsThanExpected)
{
    TTB_AUTO_TEST_CASE ("BaseTestCase_Sync_TimeoutReceivingLessEventsThanExpected");

    // We want to wait no longer than 1200 ms until all required answers
    // are received (i.e. until the configured number of sync events is
    // reached)
    TTB_SET_SYNC_TIMEOUT(1200);

    TTB_INIT_SYNC(3); // configure for receiving 3 sync events

    TTB_SYNC();       // generate the first event
    TTB_SYNC();       // generate the second event

    TTB_WAIT_SYNC();  // wait returns after the configured timeout has elapsed
                     // and generates a test error

} // BaseTestCase_Sync_TimeoutReceivingLessEventsThanExpected


//*******************************************************************


TEST_CASE(BaseTestCase_Events_FixedSequence)
{
    TTB_AUTO_TEST_CASE ("BaseTestCase_Events_FixedSequence");

    TestEvents::Get()->Act("Jack");
    TestEvents::Get()->Act("George");
    TestEvents::Get()->Act("Adam");

    // The sequence of events should be as follows:
    TTB_EXP("Jack");
    TTB_EXP("George");
    TTB_EXP("Adam");

} // BaseTestCase_Events_FixedSequence


//*******************************************************************


TEST_CASE(BaseTestCase_Events_VariableSequence)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_Events_VariableSequence");

    TestEvents::Get()->Act("Jack");
    TestEvents::Get()->Act("George");
    TestEvents::Get()->Act("Adam");

    // The sequence of events may be in any order
    TTB_EXP_VAR("Adam");
    TTB_EXP_VAR("George");
    TTB_EXP_VAR("Jack");

} // BaseTestCase_Events_VariableSequence


//*******************************************************************


TEST_CASE(BaseTestCase_Events_DetectErrorUnexpectedEventBetween)
{

    TTB_AUTO_TEST_CASE ("BaseTestCase_Events_DetectErrorUnexpectedEventBetween");

    TestEvents::Get()->Act("Result One");
    TestEvents::Get()->Act("Result XYZ (unexpected)");
    TestEvents::Get()->Act("Result Two");

    TTB_EXP ("Result One");
    TTB_EXP ("Result Two");

} // BaseTestCase_Events_DetectErrorUnexpectedEventBetween


//*******************************************************************


TEST_CASE(BaseTestCase_Events_DetectErrorUnexpectedEventsAtEnd)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_Events_DetectErrorUnexpectedEventsAtEnd");

    TestEvents::Get()->Act("Result One");
    TestEvents::Get()->Act("Result Two");
    TestEvents::Get()->Act("Result X (unexpected)");
    TestEvents::Get()->Act("Result Y (unexpected)");

    TTB_EXP ("Result One");
    TTB_EXP ("Result Two");

} // BaseTestCase_Events_DetectErrorUnexpectedEventsAtEnd


//*******************************************************************


TEST_CASE(BaseTestCase_Events_DetectErrorUnexpectedEventsAtEndWithMultipleTestCases)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_Events_DetectErrorUnexpectedEventsAtEndWithMultipleTestCases_1");
    TestEvents::Get()->Act("Result One");
    TestEvents::Get()->Act("Result Two");


    TTB_TEST_CASE("BaseTestCase_Events_DetectErrorUnexpectedEventsAtEndWithMultipleTestCases_2");
    TestEvents::Get()->Act("Result X (unexpected)");

    TTB_TEST_CASE("BaseTestCase_Events_DetectErrorUnexpectedEventsAtEndWithMultipleTestCases_3");
    TestEvents::Get()->Act("Result Y (unexpected)");
    TTB_EXP ("Result One");
    TTB_EXP ("Result Two");

} // BaseTestCase_Events_DetectErrorUnexpectedEventsAtEndWithMultipleTestCases


//*******************************************************************


TEST_CASE(BaseTestCase_Events_DetectErrorWithExplicitCheckForUnexpectedEvents)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_Events_DetectErrorWithExplicitCheckForUnexpectedEvents");

    TestEvents::Get()->Act("Result One");
    TestEvents::Get()->Act("Result Two");
    TestEvents::Get()->Act("Result X (unexpected)");
    TestEvents::Get()->Act("Result Y (unexpected)");

    TTB_EXP ("Result One");
    TTB_EXP ("Result Two");

    TTB_CHECK_FOR_UNEXPECTED_EVENTS();

} // BaseTestCase_Events_DetectErrorWithExplicitCheckForUnexpectedEvents


//*******************************************************************


TEST_CASE(BaseTestCase_ErrorEvents_Generate)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_ErrorEvents_Generate");

//    TestEvents::Get()->LogCallsToAct(true);

    // Initially there are no stored errors
    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("No errors stored");

    // Here we create some errors (usually done from your production code using the specific error
    // creation facility which has to be connected with the test environment;
    // nevertheless at the end method ErrorEvents::CreateError() is called)
    MyErrorEvents::Get()->CreateError("First error generated from test environment", 45);
    MyErrorEvents::Get()->CreateError("Another error from test environment");

    // TODO:
    // add config possibility to immediately report errors in moment of creation
    // as simple info message

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("First error generated from test environment");
    TTB_EXP ("Another error from test environment");

    // After the report all stored errros are cleared (configurable default setting)
    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("No errors stored");

} // BaseTestCase_ErrorEvents_Generate


//*******************************************************************


TEST_CASE(BaseTestCase_ErrorEvents_Chains)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_ErrorEvents_Chains");

//    TestEvents::Get()->LogCallsToAct(true);

    // In this test case we want to switch off automatic deletion of errors
    // after reporting them because we want to change the error chain hierarchy
    // of the existing errors and repeat the error report
    MyErrorEvents::Get()->DeleteStoredErrorsAfterReport(false);

    // Initially there are no stored errors
    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("No errors stored");

    // Here we create some errors (usually done from your production code using the specific error
    // creation facility which has to be connected with the test environment;
    // nevertheless at the end method ErrorEvents::CreateError() is called)
    MyErrorEvents::Get()->CreateError("Error One",   4345, 17);
    MyErrorEvents::Get()->CreateError("Error Two",   4556, 18);
    MyErrorEvents::Get()->CreateError("Error Three", 2355, 19);
    MyErrorEvents::Get()->CreateError("Error Four",  2355, 20);

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("Error Two");
    TTB_EXP ("Error Three");
    TTB_EXP ("Error Four");

    MyErrorEvents::Get()->AddAsChildError(17,18);
    MyErrorEvents::Get()->AddAsChildError(18,19);

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    MyErrorEvents::Get()->AddAsChildError(20,18);
    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("Error Four");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");

    MyErrorEvents::Get()->AddAsChildError(17,19);
    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Three");
    TTB_EXP ("Error Four");
    TTB_EXP ("  Error Two");

    // Reset to standard conditions
    MyErrorEvents::Get()->DeleteStoredErrorsAfterReport(true);
    MyErrorEvents::Get()->DeleteStoredErrors();

} // BaseTestCase_ErrorEvents_Chains


//*******************************************************************


TEST_CASE(BaseTestCase_ErrorEvents_Configuration)
{
    TTB_AUTO_TEST_CASE("BaseTestCase_ErrorEvents_Configuration");

    // Initially there are no stored errors
    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("No errors stored");

    // Configure to generate a sync event for each detected error
    MyErrorEvents::Get()->SetErrorSync(true);

    // Prepare to receive two sync events generated when errors are created
  	TTB_INIT_SYNC(2);

    MyErrorEvents::Get()->CreateError("First error",23);
    MyErrorEvents::Get()->CreateError("Second error",27);

    // Wait until the errors have been created (of course in this simple sample code the errors
    // have been created the lines above, but in a real test situation the errors may be created
    // within several threads and waiting for this to occur makes more sense
    TTB_WAIT_SYNC();

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("First error");
    TTB_EXP ("Second error");


//    TestEvents::Get()->LogCallsToAct(true);


    // Test: error msg number is configured to be ignored

    // Configure to ignore error msg num 23
    MyErrorEvents::Get()->SetNotificationActionForErrorMsgNum(23,ACTION_IGNORE);

  	TTB_INIT_SYNC(1);
    MyErrorEvents::Get()->CreateError("First error",23);
    MyErrorEvents::Get()->CreateError("Second error",27);
    TTB_WAIT_SYNC();

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Second error");


    // Test: errors within the chain may be configured to be ignored
    // their child errors will still be reported

    MyErrorEvents::Get()->SetErrorSync(false);
    MyErrorEvents::Get()->DeleteStoredErrorsAfterReport(false);

    MyErrorEvents::Get()->CreateError("Error One",   4345, 17);
    MyErrorEvents::Get()->CreateError("Error Two",   4556, 18);
    MyErrorEvents::Get()->CreateError("Error Three", 2355, 19);
    MyErrorEvents::Get()->CreateError("Error Four",  2355, 20);

    MyErrorEvents::Get()->AddAsChildError(17,18);
    MyErrorEvents::Get()->AddAsChildError(18,19);

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    MyErrorEvents::Get()->SetNotificationActionForErrorMsgNum(4556,ACTION_IGNORE);

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");


    MyErrorEvents::Get()->SetNotificationActionForErrorMsgNum(4345,ACTION_IGNORE);

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    MyErrorEvents::Get()->SetNotificationActionForErrorMsgNum(4556,ACTION_NOT_SPECIFIED);
    MyErrorEvents::Get()->SetNotificationActionForErrorMsgNum(4345,ACTION_NOT_SPECIFIED);

    MyErrorEvents::Get()->ReportStoredErrors();
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    // TODO:
    // Implement configuration possibility for category and severity
    // combine resulting instruction by building max of given notification
    // enums

    // Add possibility to write msg number, category and severity to output as part
    // of the test event

    // ACTION_INFO should only be treated as BOOST_MESSAGE
    // add connection to IMsgReceiver?

    // ACTION_ERROR_BREAK_TEST_CASE should in addition report as TestEvent
    // cause a call of BOOST_FAIL to stop execution of test case

    // Final check with boost macros results from TestToolBox::TestEvents 
    TTB_BOOST_CHECK_FOR_TEST_EVENTS();

} // BaseTestCase_ErrorEvents_Configuration

void DummyFunc(int in_val)
{
}

TEST_CASE(SearchMemLeak)
{
    cout << "TestCase_WaitForSingleAsyncAnswer Begin" << endl;

    boost::thread_group threads;
    boost::thread* pthread = threads.create_thread(
        boost::bind(&DummyFunc, 17));
threads.join_all();
// threads.remove_thread(pthread);

/*
	// Setup TestEnvironment,
    // Cleanup is automatically performed within destructor
    AutoSetupDefaultTestEnv a;

    // Instantiate the object to test
    SimpleObject so;

    // Instantiate a callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);


    // Prepare to receive a single sync event
  	TTB_INIT_SYNC(1);

    // Start a calculation
    // The result is asynchronpusly returned via callback interface
    // ICallBackFromSimpleObject::Result.
    so.CalcAsync(7);

    // Wait until the calculation has asynchronously returned the result
    TTB_WAIT_SYNC();

    // When the sync event has occured the result should be available
    // It was stored as test event (see call of TestEvents::Get()->Act
    // within CallBackFromSimpleObject::Result)
    TTB_EXP("CallBack-Result: 14");
*/
    cout << "TestCase_WaitForSingleAsyncAnswer End" << endl;

}

//*******************************************************************
// Setup the sequence of test cases (in case of manual registration)
//*******************************************************************


#ifndef AUTO_REGISTER

// Manually define the sequence of test cases to execute
test_suite* init_unit_test_suite( int, char* [] )
{
    test_suite* allTests = BOOST_TEST_SUITE( "All my test suites" );
// allTests->add( BOOST_TEST_CASE( &SearchMemLeak ));

    test_suite* testAsync= BOOST_TEST_SUITE( "Testing asynchronous reactions" );
    allTests->add(testAsync);

    testAsync->add( BOOST_TEST_CASE( &TestCase_WaitForSingleAsyncAnswer ));                   // count of failures
                                                                                              // optional: max
    testAsync->add( BOOST_TEST_CASE( &TestCase_WaitForMultipleAsyncAnswers ));
    testAsync->add( BOOST_TEST_CASE( &TestCase_VariableSequenceOfReactions ));
    if (!TTB::TheEnvironment()->IsExistingCommandLineOption("-suppressErrorTestCases"))
    {
        testAsync->add( BOOST_TEST_CASE( &TestCase_DetectMultipleErrors ),                                         2);
        testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Sync_ReceivingMoreEventsThanExpected ),                     3);
        testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Sync_TimeoutReceivingLessEventsThanExpected ),              1);
    }

    testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Events_FixedSequence ));
    testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Events_VariableSequence ));
    if (!TTB::TheEnvironment()->IsExistingCommandLineOption("-suppressErrorTestCases"))
    {
        testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Events_DetectErrorUnexpectedEventBetween ),                    1);
        testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Events_DetectErrorUnexpectedEventsAtEnd ),                     1);
        testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Events_DetectErrorUnexpectedEventsAtEndWithMultipleTestCases), 1);
        testAsync->add( BOOST_TEST_CASE( &BaseTestCase_Events_DetectErrorWithExplicitCheckForUnexpectedEvents ),      1);
    }
    test_suite* testError= BOOST_TEST_SUITE( "Testing error chains" );
    if (TTB::TheEnvironment()->IsExistingCommandLineOption("-suppressErrorTestCases"))
    {
        allTests->add(testError,0);
    }
    else
    {
        allTests->add(testError,9); // expect at least the given number of errors
    }

    testError->add( BOOST_TEST_CASE( &BaseTestCase_ErrorEvents_Generate ));
    testError->add( BOOST_TEST_CASE( &BaseTestCase_ErrorEvents_Chains ));
    testError->add( BOOST_TEST_CASE( &BaseTestCase_ErrorEvents_Configuration ));

    std::cout << "CmdLineOptions : " << TTB::TheEnvironment()->GetArgsStr()
        << std::endl << std::endl;

    return allTests;
}

#endif // not defined AUTO_REGISTER


///\endcond do not document

///@} // end group GrpTestExtensionsForBoost
