////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Class CallBackFromSimpleObjectis a call back object with interface
/// ICallBackFromSimpleObject to receive calls from SimpleObject.
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "SourcesToTest\ISimpleObject.h" // ICallBackFromSimpleObject
#include "TestToolBox/Synchronizer.h"   // synchronization via TU_SYNC
#include "TestToolBox/TestEvents.h"     // signalling (asynchronous) textual events
using namespace TestToolBox;

///\cond do not document

class CallBackFromSimpleObject :
    public ICallBackFromSimpleObject
{
public:

    // Constructor
    explicit CallBackFromSimpleObject(){};

    // Destructor
    virtual ~CallBackFromSimpleObject(){};

    //********************************************************************
    // Interface ICallBackFromSimpleObject
    //********************************************************************

    void Result (
        long in_val)
    {
        TRCF("CallBackFromSimpleObject::Result")
        TRC (TL_DEBUG, "Begin in_val = %d", in_val);

        // Inform test utility about the received value
        TestEvents::Get()->Act("CallBack-Result: %d", in_val);

        // TODO / possible extension:
        // Add implementation variant allowing output to stream instead of printf syntax

        // Generate sync event to synchronize with main thread of test exe
        TTB_SYNC();

        TRC (TL_DEBUG, "End");
    }

}; // class CallBackFromSimpleObject

///\endcond do not document
