#include "stdafx.h"
#include "TestToolBox/Test/TestRunner/CallbackFromSimpleObject.h"
#include "TestToolBox/CommonSources/CommonDefinitions.h"

//-----------------------------------------------------------------------------

namespace TestToolBox
{
    // forward declarations
    std::string MakeReturnValue(double& out_retVal, bool in_simulateFailure, std::string const & in_methodName, SimulatedObjectBase const * in_pObject);
    std::string MakeReturnValue(int& out_retVal, bool in_simulateFailure, std::string const & in_methodName, SimulatedObjectBase const * in_pObject);
    std::string MakeReturnValue(SomeErrorResult& out_retVal, bool in_simulateFailure, std::string const & in_methodName, SimulatedObjectBase const * in_pObject);
}

//-----------------------------------------------------------------------------

CallBackFromSimpleObject::CallBackFromSimpleObject(
    bool in_generateSyncEvent)
    : RegisteredSimulatedObject ("CbSimpleObject", TTB_CLASS_NAME)
{
    if (in_generateSyncEvent)
    {
        _SetOption(Option::oSYNC);
    }
}

//-----------------------------------------------------------------------------

void CallBackFromSimpleObject::Result (
    long in_val)
{
    TRCF("CallBackFromSimpleObject::Result")
        TRC (TL_DEBUG, "Begin in_val = %d", in_val);

    // Inform test utility about the received value
    TTB_ACT_S("CallBack-Result: " << in_val);

    // Generate sync event to synchronize with main thread of test exe
    if (_IsOptionSet(Option::oSYNC))
    {
        TTB_SYNC();
    }

    TRC (TL_DEBUG, "End");
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultA(
    long in_value, long& out_value)
{
    // assume regular processing
    int retVal = 0; // no error
    out_value = 4711;

    _MethodCall(__FUNCTION__,

        // Log input params
        TTB_STRING_S("in_val=" << in_value),

        // Delayed processing of output/return values
        ([&](void) -> std::string
    {
        std::ostringstream oss;
        if (_IsError("SpecialResultA"))
        {
            retVal = -1;
            oss << "return error code =" << retVal;
        }
        else
        {
            oss << "out_value=" << out_value;
        }
        return oss.str();
    }));

    return retVal;
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultB(
    long in_value, long& out_value)
{
    // assume regular processing
    int retVal = 0; // no error
    out_value = 4712;

            // int billy [] = { 16, 2, 77, 40, 12071 };

    TTB_METHOD_CALL2("in_val=" << in_value,

        //  Remark: macro TTB_METHOD_CALL2 supports easy programming of lambda functor but has
        // the following disadvantages:
        // (1) "," within legal C++ code of lambda functor leads to macro errors
        // (2) debugging the lambda code line by line or setting break points is not possible,
        // because from the compilers point of view the whole code is defined within the line of the macro definition
        // Recommendation: to be used only for simple code lines
        // possibly delayed processing:
        std::ostringstream oss;
        if (_IsError("SpecialResultB"))
        {
            retVal = -1;
            oss << "return error code =" << retVal;
        }
        else
        {
            oss << "out_value=" << out_value;
        }
        return oss.str();
);

    return retVal;
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultC(
    long in_value, long& out_value)
{
    // assume regular processing
    int retVal = 0; // no error
    out_value = 4713;

    _MethodCall(__FUNCTION__,

        // Log input params
        TTB_STRING_S("in_val=" << in_value),

        // Delayed processing of output/return values
        ([&](void) -> std::string
    {
        std::ostringstream oss;
        if (_IsError("SpecialResultC"))
        {
            retVal = -1;
            oss << "return error code =" << retVal;
        }
        else
        {
            oss << "out_value=" << out_value;
        }
        return oss.str();
    }));

    return retVal;
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultD(
    long in_value, long& out_value)
{
    // assume simple implementation is enough, method name is added automatically,
    // full support for standard options oSILENT, oBLOCKING, oNO_SYNC, oNO_OBJECT_NAME comes from base implementation!
    TTB_METHOD_CALL();
    return 0;
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultE(
    long in_value, long& out_value)
{
    // you can define any reaction and set output params and return value as you desire.
    // To make decisions you can check all options from base class and use any properties
    // introduced in your derived class.
    // The only thing which is not possible here is to perform a delayed evaluation
    // of results when in blocked call state a continue of method execution follows
    out_value = 4715;

    // method name is added automatically
    // you can easily add info about call params and returned values:
    TTB_METHOD_CALL1("in_value=" << in_value << " out_value=" << out_value);

    // here you can check the conditions which are valid in the moment of continuing a blocked function call
    // but ther is no possibility to add a corresponding info to the output line of the function call
    out_value = 4747; // not visible in output

    // of cource you can produce here an extra line of output with the valid result, but usually this is not the
    // output you desire. To support delayed evaluation and writing delayed results to output please use
    // macro TTB_METHOD_CALL2 or _MethodCall with all 3 possible params.
    return 0;
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultF(
    long in_value, long& out_value)
{
    out_value = 4711;

    return _MethodCallReturn<int>(__FUNCTION__,
        TTB_STRING_S("in_val=" << in_value));
}

//-----------------------------------------------------------------------------

int CallBackFromSimpleObject::SpecialResultG(
    long in_value, long& out_value)
{
    out_value = 4711;

    return TTB_METHOD_CALL_RET2(int, "in_val=" << in_value);
}

//-----------------------------------------------------------------------------

SomeErrorResult CallBackFromSimpleObject::SpecialResultH(
    long in_value, long& out_value)
{
    out_value = 4711;

    return TTB_METHOD_CALL_RET1(SomeErrorResult);
}

//-----------------------------------------------------------------------------
