////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Interface ISimpleObject with callback ICallBackFromSimpleObject is used
/// within demo test application "TestRunner"
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once
///\cond do not document

#include <string>

    struct SomeErrorResult
    {
        SomeErrorResult ()
            : m_errorCode (0)
        {}

        SomeErrorResult (
            int                 in_errorCode,
            std::string const & in_errorInfo)

            : m_errorCode (in_errorCode)
            , m_errorInfo (in_errorInfo)
        {}

        int         m_errorCode;
        std::string m_errorInfo;
    };

// Calback interface
struct ICallBackFromSimpleObject
{
    // Used for synchronous and asynchronous callbacks
    virtual void Result(
        long in_value) = 0;

    // Used for synchronous and asynchronous callbacks
    virtual int SpecialResultA(
        long in_value, long& out_value) = 0;
    virtual int SpecialResultB(
        long in_value, long& out_value) = 0;
    virtual int SpecialResultC(
        long in_value, long& out_value) = 0;
    virtual int SpecialResultD(
        long in_value, long& out_value) = 0;
    virtual int SpecialResultE(
        long in_value, long& out_value) = 0;
    virtual int SpecialResultF(
        long in_value, long& out_value) = 0;
    virtual int SpecialResultG(
        long in_value, long& out_value) = 0;
    virtual SomeErrorResult SpecialResultH(
        long in_value, long& out_value) = 0;
};

// Interface to the object under test
struct ISimpleObject
{
    // Establishes connection using custom callback interface
    virtual void Connect(
        ICallBackFromSimpleObject* in_pICallBack) = 0;

    // Returns result of calculation via synchronous callback
    virtual void CalcSync(
        long in_val) = 0;

    // Returns result of calculation via asynchronous callback
    virtual void CalcAsync(
        long in_val) = 0;

    // Returns result of calculation via synchronous callback
    // and return possible error as function return value.
    virtual long CalcSyncAndReturnErrorCode (
        long in_val) = 0;

};

///\endcond do not document
