////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of class SimpleObject
/// 
////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include <windows.h>
#include "SimpleObject.h"
#include <boost/bind.hpp>
#include "TestToolBox/CommonSources/CommonDefinitions.h"
#include "TestToolBox/Runner.h" // to be able to create an error of type MyErrors


///\cond do not document


//************************************************************************
//
// Interface ISimpleObject
//
//************************************************************************


void SimpleObject::Connect(
    ICallBackFromSimpleObject* in_pICallBack)
{
    // Store callback interface to client
	m_pICallBack = in_pICallBack;

} // SimpleObject::Connect


//************************************************************************
//************************************************************************


void SimpleObject::CalcSync(long in_val)
{
    TRCF("CalcSync");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    ExecCalc(in_val);
    //if (m_pICallBack)
    //{
    //    if (in_val >= 0)
    //    {
    //        m_pICallBack-> Result (InternalCalc (in_val));
    //    }
    //    else
    //    {
    //        long outVal;
    //        int errorCode = m_pICallBack-> NegativeResult (InternalCalc (in_val), outVal);
    //    }
    //}

    TRC(TL_DEBUG,"End");

} // SimpleObject::CalcSync


//************************************************************************
//************************************************************************


void SimpleObject::CalcAsync (long in_val)
{
    TRCF("CalcAsync");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    // Execute asynchronously within separate thread
    m_threads.create_thread(
        boost::bind(&SimpleObject::ExecCalc,this, in_val));

    TRC(TL_DEBUG,"End");

} // SimpleObject::CalcAsync


//************************************************************************
//************************************************************************


long SimpleObject::CalcSyncAndReturnErrorCode(long in_val)
{
    TRCF("CalcSyncAndReturnErrorCode");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    long retVal = 0;
    if (in_val == 301)
    {
        TestToolBox::MyErrorEvents::Get()->CreateError("General calculation error",
            1001, 4711);
        retVal = 4711;
    }
    else if (in_val == 302)
    {
        TestToolBox::MyErrorEvents::Get()->CreateError("Wrong input value",
            1002, 4734);
        TestToolBox::MyErrorEvents::Get()->CreateError("Value to big",
            1004, 4736);
        TestToolBox::MyErrorEvents::Get()->CreateError("Calculation cancelled",
            1005, 4789);
        TestToolBox::MyErrorEvents::Get()->CreateError("Range exceeded",
            1003, 4754);

        TestToolBox::MyErrorEvents::Get()->AddAsChildError(4734,4754);
        TestToolBox::MyErrorEvents::Get()->AddAsChildError(4754,4736);
        TestToolBox::MyErrorEvents::Get()->AddAsChildError(4734,4789);

        retVal = 4734;
    }

    // Only callback when no error is simulated
    if (retVal == 0 && m_pICallBack)
    {
        m_pICallBack-> Result (InternalCalc (in_val));
    }

    TRC(TL_DEBUG,"End retVal=%d", retVal);
    return retVal;

} // SimpleObject::CalcSyncAndReturnErrorCode


//************************************************************************
// Other member functions
//************************************************************************

long SimpleObject::InternalCalc (long in_val)
{
    // Remark: Here we directly create the errors in the format used within TTB.
    // A real world application has its own error format which has to be
    // transformed in the TTB format.

    if (in_val == 101)
    {
        TestToolBox::MyErrorEvents::Get()->CreateError("Error One",   4345, 17);
    }
    else if (in_val == 102)
    {
        TestToolBox::MyErrorEvents::Get()->CreateError("Error One",   4345, 17);
        TestToolBox::MyErrorEvents::Get()->CreateError("Error Two",   4556, 18);
        TestToolBox::MyErrorEvents::Get()->CreateError("Error Three", 2355, 19);
        TestToolBox::MyErrorEvents::Get()->CreateError("Error Four",  2357, 20);

        TestToolBox::MyErrorEvents::Get()->AddAsChildError(17,18);
        TestToolBox::MyErrorEvents::Get()->AddAsChildError(18,19);
    }

    return in_val * 2;

} // SimpleObject::InternalCalc


//************************************************************************
//************************************************************************


void SimpleObject::ExecCalc (long in_val)
{

    TRCF("ExecCalc");
    TRC(TL_DEBUG,"Begin:  in_val = %d", in_val);

    if (m_pICallBack)
    {
        if (in_val >= 0)
        {
            m_pICallBack-> Result (InternalCalc (in_val));
        }
        else if (in_val >-10)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultA (InternalCalc (in_val), outVal);
        }
        else if (in_val >-20)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultB (InternalCalc (in_val), outVal);
        }
        else if (in_val >-30)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultC (InternalCalc (in_val), outVal);
        }
        else if (in_val >-40)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultD (InternalCalc (in_val), outVal);
        }
        else if (in_val >-50)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultE (InternalCalc (in_val), outVal);
        }
        else if (in_val >-60)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultF (InternalCalc (in_val), outVal);
        }
        else if (in_val >-70)
        {
            long outVal;
            int errorCode = m_pICallBack-> SpecialResultG (InternalCalc (in_val), outVal);
        }
        else if (in_val >-80)
        {
            long outVal;
            SomeErrorResult err = m_pICallBack-> SpecialResultH (InternalCalc (in_val), outVal);
        }
    }

    TRC(TL_DEBUG,"End");

} // SimpleObject::ExecCalc


//************************************************************************
//************************************************************************

///\endcond do not document
