#include "stdafx.h"
#include "TestToolBox/TestToolBox.h"
#include <iostream>
#include <iomanip>

using namespace std;
using namespace TestToolBox;

namespace
{
    void GenerateSequenceOfEvents (void)
    {
        // Simply generate some events
        // assuming that 3 threads generate both identical
        // and distinct event entries
        // In a real world test application the events may be generated by several 
        // simulated objects communicating with the object under test
        TheTestEvents()->Act("First Number");               // assume thread 3
        TheTestEvents()->Act("Second number of thread 3");  // assume thread 3
        TheTestEvents()->Act("Some other event");
        TheTestEvents()->Act("First Number");               // assume thread 1
        TheTestEvents()->Act("Third number");               // assume thread 3
        TheTestEvents()->Act("Second number of thread 1");  // assume thread 1
        TheTestEvents()->Act("First Number");               // assume thread 2
        TheTestEvents()->Act("Some other event");
        TheTestEvents()->Act("Some other event");
        TheTestEvents()->Act("Second number of thread 2");  // assume thread 2
        TheTestEvents()->Act("Third number");               // assume thread 1
        TheTestEvents()->Act("Third number");               // assume thread 2
        TheTestEvents()->Act("Fourth number of thread 1");  // assume thread 1
        TheTestEvents()->Act("Some other event");
        TheTestEvents()->Act("Fourth number of thread 2");  // assume thread 2
        // You can also use stream syntax:
        TTB_ACT_S           ("Fourth number of thread "  << 3);  // assume thread 3
    }
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckForMultipleSequences_StepA)
{
    GenerateSequenceOfEvents();

    // Check some events not part of the sequences
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");

    // Simply check that the following sequences have occured in the given order

    TheTestEvents()->DefineSequence("One");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 1");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 1");

    TheTestEvents()->DefineSequence("Two");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 2");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 2");

    TheTestEvents()->DefineSequence("Three");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 3");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 3");

    TTB_CHECK_DEFINED_SEQUENCES();

    // Check some events not part of the sequneces
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (
	TestEventsCheckForMultipleSequences_ErrorDetection ,"TestEventsCheckForMultipleSequences (ErrorDetection)")
{
    if (!TheEnvironment()->IsExistingCommandLineOption("-showErrorDetection"))
    {
        TheProtocol()->Writeln(OutputLevel::eERROR, OutputMode::M_ALL,
            "SKIPPING test, activate with option -showErrorDetection");
        return;
    }

    GenerateSequenceOfEvents();

    // Check some events not part of the sequences
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");

    TheTestEvents()->DefineSequence("Three");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 3");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 3");

    TheTestEvents()->DefineSequence("One");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 1");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 1");

    TheTestEvents()->DefineSequence("Two");
    TTB_EXP ("First Number");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 2");
    TTB_EXP ("Second number of thread 2"); // impossible order which will cause an error

    TTB_CHECK_DEFINED_SEQUENCES();

    // Check some events not part of the sequneces
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckForMultipleSequences_StepB)
{
    GenerateSequenceOfEvents();

    // Simply check that the following sequences have occured in the given order
    // The sequences are defined in a slightly changed order

    TheTestEvents()->DefineSequence("Three");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 3");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 3");

    TheTestEvents()->DefineSequence("One");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 1");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 1");

    TheTestEvents()->DefineSequence("Two");
    TTB_EXP ("First Number");
    TTB_EXP ("Second number of thread 2");
    TTB_EXP ("Third number");
    TTB_EXP ("Fourth number of thread 2");

    TTB_CHECK_DEFINED_SEQUENCES();

    // Check some events not part of the sequneces
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");
    TTB_EXP_VAR("Some other event");
}

//-----------------------------------------------------------------------------
