#include "stdafx.h"
#include "TestToolBox/Runner.h"
#include "TestToolBox/TestEvents.h"
#include <iostream>
#include <iomanip>
//#include "assert.h"

using namespace std;
using namespace TestToolBox;


namespace TestToolBox
{

}

void GenerateSequenceOfEvents (void)
{
    // Simply generate some events
    // In a real world test application the events may be generated by several 
    // simulated objects communicating with the object under test
    TheTestEvents()->Act("Number: 2");
    TheTestEvents()->Act("Number: 4");
    TheTestEvents()->Act("double value: 88.88");
    TheTestEvents()->Act("Number: 6");
    TheTestEvents()->Act("Number: 8");
    TheTestEvents()->Act("double value: 66.66");
    TheTestEvents()->Act("Number: 10");
    TheTestEvents()->Act("double value: 44.44");
    TheTestEvents()->Act("double value: 22.22");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckForExactSequence)
{
	GenerateSequenceOfEvents();

    // Simply check that the sequence has occured in exactly the given order
    TTB_EXP ("Number: 2");
    TTB_EXP ("Number: 4");
    TTB_EXP ("double value: 88.88");
    TTB_EXP ("Number: 6");
    TTB_EXP ("Number: 8");
    TTB_EXP ("double value: 66.66");
    TTB_EXP ("Number: 10");
    TTB_EXP ("double value: 44.44");
    TTB_EXP ("double value: 22.22");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckForArbitrarySequence)
{
    GenerateSequenceOfEvents();

    // Check that all events have occured. There is no assumption about the occured order
    TTB_EXP_VAR ("Number: 2");
    TTB_EXP_VAR ("Number: 4");
    TTB_EXP_VAR ("Number: 6");
    TTB_EXP_VAR ("Number: 8");
    TTB_EXP_VAR ("Number: 10");
    TTB_EXP_VAR ("double value: 22.22");
    TTB_EXP_VAR ("double value: 44.44");
    TTB_EXP_VAR ("double value: 66.66");
    TTB_EXP_VAR ("double value: 88.88");

    GenerateSequenceOfEvents();

    // Any sequence is allowed
    TTB_EXP_VAR ("double value: 66.66");
    TTB_EXP_VAR ("Number: 2");
    TTB_EXP_VAR ("Number: 8");
    TTB_EXP_VAR ("Number: 10");
    TTB_EXP_VAR ("double value: 22.22");
    TTB_EXP_VAR ("double value: 44.44");
    TTB_EXP_VAR ("double value: 88.88");
    TTB_EXP_VAR ("Number: 4");
    TTB_EXP_VAR ("Number: 6");


    // Alternatively you can also use TTB_EXP and set the required CheckMode
    TheTestEvents()->SetCheckType(CheckType::eVARIABLE);

    GenerateSequenceOfEvents();

    TTB_EXP ("double value: 66.66");
    TTB_EXP ("Number: 2");
    TTB_EXP ("Number: 8");
    TTB_EXP ("Number: 10");
    TTB_EXP ("double value: 22.22");
    TTB_EXP ("double value: 44.44");
    TTB_EXP ("double value: 88.88");
    TTB_EXP ("Number: 4");
    TTB_EXP ("Number: 6");

    TheTestEvents()->ResetCheckType();
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckForSequentialOrder)
{
    GenerateSequenceOfEvents();

    // Check that the small numbers have occured in increasing order
    TTB_EXP_SEQ ("Number: 2");
    TTB_EXP_SEQ ("Number: 4");
    TTB_EXP_SEQ ("Number: 6");
    TTB_EXP_SEQ ("Number: 8");
    TTB_EXP_SEQ ("Number: 10");

    // Goto back to first unchecked position within the array of events
    TheTestEvents()->ResetCurrentCheckPosition();

    // Check that the big numbers have occured in decreasing order
    TTB_EXP_SEQ ("double value: 88.88"); 
    TTB_EXP_SEQ ("double value: 66.66");
    TTB_EXP_SEQ ("double value: 44.44");
    TTB_EXP_SEQ ("double value: 22.22");


    // Alternatively you can also use TTB_EXP and set the required CheckMode
    TheTestEvents()->SetCheckType(CheckType::eSEQUENTIAL);

    GenerateSequenceOfEvents();

    // Check that the small numbers have occured in increasing order
    TTB_EXP ("Number: 2");
    TTB_EXP ("Number: 4");
    TTB_EXP ("Number: 6");
    TTB_EXP ("Number: 8");
    TTB_EXP ("Number: 10");

    // Goto back to first unchecked position within the array of events
    TheTestEvents()->ResetCurrentCheckPosition();

    // Check that the big numbers have occured in decreasing order
    TTB_EXP ("double value: 88.88"); 
    TTB_EXP ("double value: 66.66");
    TTB_EXP ("double value: 44.44");
    TTB_EXP ("double value: 22.22");

    TheTestEvents()->ResetCheckType();
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckEventPositions)
{
    // Return value of TTB_EXP macros is the found event position
    {
        GenerateSequenceOfEvents();

        int pos_2 = TTB_EXP_SEQ ("Number: 2");
        TTB_EXP_SEQ ("Number: 4");
        TTB_EXP_SEQ ("Number: 6");
        TTB_EXP_SEQ ("Number: 8");
        int pos_10 = TTB_EXP_SEQ ("Number: 10");
        TTB_EXP_VAR ("double value: 88.88"); 
        TTB_EXP_VAR ("double value: 66.66");
        int pos_4444 = TTB_EXP_VAR ("double value: 44.44");
        TTB_EXP_VAR ("double value: 22.22");

        TTB_LIST_ALL_STORED_EVENTS();

        TTB_CHECK_EQUAL (pos_10+1, pos_4444);
        TTB_CHECK_COND (pos_10 < pos_4444)
    }

    // Use more readable syntax, by using method StorePos() of the returned
    // object EventPosition or by using member function TestEvents::GetLastEventPosition()
    {
        GenerateSequenceOfEvents();

        int pos_2;
        int pos_10;
        int pos_4444;

        TTB_EXP_SEQ ("Number: 2").    StorePos(pos_2);
        TTB_EXP_SEQ ("Number: 4");
        TTB_EXP_SEQ ("Number: 6");
        TTB_EXP_SEQ ("Number: 8");
        TTB_EXP_SEQ ("Number: 10").   StorePos(pos_10);
        TTB_EXP_VAR ("double value: 88.88"); 
        TTB_EXP_VAR ("double value: 66.66");
        TTB_EXP_VAR ("double value: 44.44"); pos_4444 = TheTestEvents()->GetLastEventPosition();
        TTB_EXP_VAR ("double value: 22.22");

        TTB_CHECK_EQUAL (pos_10+1, pos_4444);
        TTB_CHECK (pos_10, std::less<int>(), pos_4444)
    }
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckWithPrintfSyntax)
{
    GenerateSequenceOfEvents();

    TTB_EXP1     ("Number: %d", 2);
    TTB_EXP2     ("%s: %d", "Number", 4);
    TTB_EXP_SEQ1 ("Number:%2d", 6);
    TTB_EXP_SEQ1 ("Number:%s", " 8");
    TTB_EXP_SEQ2 ("Number: %c%c", '1', '0');

    TTB_EXP_VAR1 ("double value: %s", "88.88"); 
    TTB_EXP_VAR1 ("double value: %.2f", 66.66);
    TTB_EXP_VAR1 ("double value: %.2f", 44.4423); // values are rounded before compare
    TTB_EXP_VAR1 ("double value:%6.2f", 22.219);  // values are rounded before compare
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (TestEventsCheckWithStreamSyntax)
{
    GenerateSequenceOfEvents();

    // Limitations:
    // Stream enabled macros do not have a return value for the found event position.
    // You have to use TestEvents::GetLastEventPosition() to retrieve the needed value.

    int pos_6;

    TTB_EXP_S     ("Number: " << 2);
    TTB_EXP_S     ("Number" << ": " << 4);
    TTB_EXP_SEQ_S ("Number:" << std::setw(2) << 6); pos_6 = TheTestEvents()->GetLastEventPosition();
    TTB_EXP_SEQ_S ("Number:" << " 8");
    TTB_EXP_SEQ_S ("Number: " << '1' << '0');

    TTB_EXP_VAR_S ("double value: " << "88.88"); 
    TTB_EXP_VAR_S ("double value: " << std::fixed << std::setprecision(2) << 66.66); 
    TTB_EXP_VAR_S ("double value: " << std::fixed << std::setprecision(2) << 44.4423); // values are rounded before compare
    TTB_EXP_VAR_S ("double value:" << std::setw(6) << std::fixed << std::setprecision(2) << 22.219); // values are rounded before compare

}

//-----------------------------------------------------------------------------
