#include "stdafx.h"
#include "TestToolBox/TestToolBox.h"

using namespace TestToolBox;

//-----------------------------------------------------------------------------

bool IsGreaterThan(int left, int right)
{return left > right;}

bool SomeBoolFunc(int i)
{return i>25;}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (CheckCondition)
{
    int i = 23;
    int j = 5;

    // Check bool expression
    TTB_CHECK_COND(i>j);
    TTB_CHECK_COND(i > 0);

    // Check bool variable
    bool someBoolVariable = true;
    TTB_CHECK_COND(someBoolVariable);

    // Check bool function
    TTB_CHECK_COND(SomeBoolFunc(31));
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (CheckCondition_ErrorDetection)
{
    if (!TheEnvironment()->IsExistingCommandLineOption("-showErrorDetection"))
    {
        TheProtocol()->Writeln(OutputLevel::eERROR, OutputMode::M_ALL,
            "SKIPPING test, activate with option -showErrorDetection");
        return;
    }

    int i = 23;
    int j = 5;

    // Check bool expression
    TTB_CHECK_COND(j>=i);
    TTB_CHECK_COND(i == 0);

    // Check bool variable
    bool someBoolVariable = false;
    TTB_CHECK_COND(someBoolVariable);

    // Check bool function
    TTB_CHECK_COND(SomeBoolFunc(12));
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (CheckEqual)
{
    int i = 23;
    int j = 5;

    // Check integer expressions
    TTB_CHECK_EQUAL(i,j+18);

    // Check bool variables
    bool someBool = true;
    bool otherBool = true;
    TTB_CHECK_EQUAL(someBool,otherBool);

    // Check bool expression
    TTB_CHECK_EQUAL(i<j, 2*i==5);

    // Check double values
    double pi = 3.14159265;
    float f = 3.1435f;
    TheTestEvents()->SetDoublePrecision(2);
    TTB_CHECK_EQUAL(pi, f);
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (CheckEqual_ErrorDetection)
{
    if (!TheEnvironment()->IsExistingCommandLineOption("-showErrorDetection"))
    {
        TheProtocol()->Writeln(OutputLevel::eERROR, OutputMode::M_ALL,
            "SKIPPING test, activate with option -showErrorDetection");
        return;
    }

    int i = 23;
    int j = 5;

    // Check integer expressions
    TTB_CHECK_EQUAL(i-4,j+18);

    // Check bool variables
    bool someBool = true;
    bool otherBool = false;
    TTB_CHECK_EQUAL(someBool,otherBool);

    // Check bool expression
    TTB_CHECK_EQUAL(i>=j, 2*i==5);

    // Check double values
    double pi = 3.14159265;
    float f = 3.1435f;
    TheTestEvents()->SetDoublePrecision(4);
    TTB_CHECK_EQUAL(pi, f);
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC (CheckWithCompareFunction)
{
    int i = 23;
    int j = 5;

    double pi = 3.14159265;
    double f = 3.1435f;

    // User defined compare function (binary predicate)
    TTB_CHECK(i, IsGreaterThan, j);

    // Using standard predicates
    TTB_CHECK(i, std::less_equal<int>(), i);
    TTB_CHECK(pi, std::less<double>(), f);
}


//-----------------------------------------------------------------------------

TTB_TEST_FUNC (CheckWithCompareFunction_ErrorDetection)
{
    int i = 23;
    int j = 5;

    double pi = 3.14159265;
    double f = 3.1435f;

    if (!TheEnvironment()->IsExistingCommandLineOption("-showErrorDetection"))
    {
        TheProtocol()->Writeln(OutputLevel::eERROR, OutputMode::M_ALL,
            "SKIPPING test, activate with option -showErrorDetection");
        return;
    }

    // User defined compare function (binary predicate)
    TTB_CHECK(j, IsGreaterThan, i);

    // Using standard predicates
    TTB_CHECK(j, std::greater<int>(), i);
    TTB_CHECK(pi, std::greater_equal<double>(), f);
}

//-----------------------------------------------------------------------------
