#include "stdafx.h"
#include "TestToolBox/Runner.h"
#include "TestToolBox/Test/TestRunner/SourcesToTest/SimpleObject.h"
#include "TestToolBox/Test/TestRunner/CallbackFromSimpleObject.h"
#include <iostream>
using namespace std;

// The name of the test function is used as the name of the test case
TTB_TEST_FUNC (WaitingForSingleAsyncAnswer)
{
    // Instantiate the object to test
    SimpleObject so;

    // Instantiate a callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);
	::Sleep(5000);
    // Prepare to receive a single sync event
  	TTB_INIT_SYNC(1);

    // Optionally you can describe your test case or the actions within it);
    TTB_INFO("Calling CalcAsync(7)\n"
             "(Waiting for asynchronous result via callback)");

    // Start a calculation
    // The result is asynchronusly returned via callback interface
    // ICallBackFromSimpleObject::Result.
    so.CalcAsync(7);

    // Wait until the calculation has asynchronously returned the result
    TTB_WAIT_SYNC();

    // When the sync event has occured the result should be available
    // It was stored as test event (see call of TestEvents::Get()->Act
    // within CallBackFromSimpleObject::Result)
    TTB_EXP("CallBack-Result: 14");

} // WaitingForSingleAsyncAnswer


//-----------------------------------------------------------------------------


// You can use more extended descriptions for your test case
TTB_TEST_FUNC_DESC (WaitingForMultipleAsyncAnswers,
    "Waiting for multiple async answers - sequence of events is strict")
{
    // Instantiate the object to test
    SimpleObject so;

    // Instantiate a callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    // Prepare to receive 3 sync events
  	TTB_INIT_SYNC(3);

    // Start 3 calculations which will return their results
    // asynchronously but in the same sequence

    // Optionally you can use macro TTB_CMD to call a method and also log
    // the call within test stream.
    // i.e. instead of writing
    // so.CalcSync(2);
    // so.CalcSync(33);
    // so.CalcSync(98);
    // both call and log using the following syntax:
    TTB_CMD1(so.CalcSync,2);
    TTB_CMD1(so.CalcSync,-33);
    TTB_CMD1(so.CalcSync,99);

    // Remarks: To keep the code of the object under test simple
    // here the synchronous execution methods are called
    // otherwise the preserving of sequence would not be guaranteed.
    // In a real world application think of a situation where you make one
    // or more interactions and then you expect three reactions in exactly
    // the specified order

    // Wait until the calculation has asynchronously returned all results
    TTB_WAIT_SYNC();

    // When the sync events have occured all results should be available
    // Remark: The sequence of macro calls has to be in the same order as the corresponding
    // calls to CalcSync otherwise an error is reported.
    TTB_EXP ("CallBack-Result: 4");
    TTB_EXP1("CallBack-Result: %d",         66);  // example of using printf syntax
    TTB_EXP2("CallBack-%s: %d" , "Result", 198);  // example of using printf syntax

} // WaitingForMultipleAsyncAnswers


//-----------------------------------------------------------------------------


TTB_TEST_FUNC_DESC (WaitingForMultipleAsyncAnswersWithVariableSequence,
    "Waiting for multiple async answers, sequence of events may change randomly")
{
    // Instantiate the object to test
    SimpleObject so;

    // Instantiate an callback object and connect with the object to test
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    // Now execute some calculations.
    // The answers may come in any order.

  	TTB_INIT_SYNC(4);

    TTB_INFO("Calling CalcAsync(1,2,3,4)\n"
             "(Waiting for asynchronous results via callback)");

    // Start 4 calculations
    so.CalcAsync(1);
    so.CalcAsync(2);
    so.CalcAsync(3);
    so.CalcAsync(4);

    TTB_WAIT_SYNC();

    // When the sync events have occured all results should be available
    // Remark: The following macro calls indicate that the sequence of
    // events does not matter
    TTB_EXP_VAR ("CallBack-Result: 6");
    TTB_EXP_VAR ("CallBack-Result: 4");
    TTB_EXP_VAR ("CallBack-Result: 2");
    TTB_EXP_VAR ("CallBack-Result: 8");

} // WaitingForMultipleAsyncAnswersWithVariableSequence


//-----------------------------------------------------------------------------
