#include "stdafx.h"
#include "TestToolBox/TestToolBox.h"
#include "TestToolBox/Test/TestRunner/SourcesToTest/SimpleObject.h"
#include "TestToolBox/Test/TestRunner/CallbackFromSimpleObject.h"
#include <iostream>
using namespace std;
namespace TTB = TestToolBox;
using namespace TestToolBox::ShortNames;

TTB_TEST_FUNC (GroupBSomeTestWithDuplicateName_Errors)
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb(false);
    so.Connect(&cb);

    //---------------------------------------------------------------
    TTB_SECTION("Basic checks for error chains");
    //---------------------------------------------------------------

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("No errors stored");

    TTB_INFO ("\nA single error is created");
    TTB_CMD1 (so.CalcSync, 101);
    TTB_EXP  ("CallBack-Result: 202");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");

    TTB_INFO ("\nAn error chain with several errors is created");
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    TTB_INFO("\nA second check of errors results in an empty list"
             "\n because per default the errors are automatically cleared after reporting");
    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("No errors stored");


    //---------------------------------------------------------------
    TTB_SECTION("Ignoring specific errors");
    //---------------------------------------------------------------

    TTB_CMD2 (SetIgnoreError, 4556, true);
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");


    TTB_CMD2 (SetIgnoreError, 4345, true);
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");


    TTB_CMD2 (SetIgnoreError, 4345, false);
    TTB_CMD2 (SetIgnoreError, 4556, false);
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");


    //---------------------------------------------------------------
    TTB_SECTION("Ignoring errors containing specific text");
    //---------------------------------------------------------------

    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    TTB_CMD1 (IgnoreErrorContainingTextPattern, "wo");

    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    TTB_CMD1 (IgnoreErrorContainingTextPattern, "r One");

    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    TTB_CMD (ClearStoredErrorTextPatterns);
 
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    //---------------------------------------------------------------
    TTB_SECTION("Activation and deactivation of immediate logging of errors within protcol file");
    //---------------------------------------------------------------

    TTB_INFO ("You can deactivate immediate logging of errors as info within protocol file");
    TTB_CMD1 (SetImmediateErrorLog, false);
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (ReportStoredErrors);
    TTB_EXP ("Error One");
    TTB_EXP ("  Error Two");
    TTB_EXP ("    Error Three");
    TTB_EXP ("Error Four");

    TTB_INFO ("The errors are reported again as info within protocol file");
    TTB_CMD1 (SetImmediateErrorLog, true);
    TTB_CMD1 (so.CalcSync, 102);
    TTB_EXP  ("CallBack-Result: 204");

    TTB_CMD (DeleteStoredErrors); // we do not need the errors here, for effect see protocol file


    //---------------------------------------------------------------
    TTB_SECTION("Before the dangerous call");
    //---------------------------------------------------------------

    TTB_INFO("Depending on the command line args an exception may be thrown\n"
             "to test the capability of the Runner environment to catch it.");

	if (TTB::Environment::Get()->IsExistingCommandLineOption("-testOfExceptions"))
	{
		throw std::exception("Some fatal error");
	}

    //---------------------------------------------------------------
    TTB_SECTION("After the dangerous call");
    //---------------------------------------------------------------

    TTB_INFO("When we are here, no exception was thrown");
}

namespace AnyNamespaceThatAllowsUsingOfSameFunctionName
{
	TTB_TEST_FUNC (GroupBSomeTestWithDuplicateName_Errors)
	{
        // Prepare for testing
        SimpleObject so;
        CallBackFromSimpleObject cb;
        so.Connect(&cb);

        //---------------------------------------------------------------
        TTB_SECTION("Errors can rise sync events");
        //---------------------------------------------------------------

        TTB_INFO("Per default errors do not rise sync evetns");

      	TTB_INIT_SYNC(1); // only callback rises sync event
        TTB_CMD1(so.CalcAsync, 102);
        TTB_WAIT_SYNC();

        TTB_EXP ("CallBack-Result: 204");

        TTB_CMD (ReportStoredErrors);
        TTB_EXP ("Error One");
        TTB_EXP ("  Error Two");
        TTB_EXP ("    Error Three");
        TTB_EXP ("Error Four");


        TTB_INFO("Now activate sync events for error creation");

        TTB_CMD1(SetErrorSync, true);
      	TTB_INIT_SYNC(5); // callback and 4 errors
        TTB_CMD1(so.CalcAsync, 102);
        TTB_WAIT_SYNC();

        TTB_EXP ("CallBack-Result: 204");

        TTB_CMD (ReportStoredErrors);
        TTB_EXP ("Error One");
        TTB_EXP ("  Error Two");
        TTB_EXP ("    Error Three");
        TTB_EXP ("Error Four");

        TTB_CMD1(SetErrorSync, false);


        //---------------------------------------------------------------
        TTB_SECTION("Checking errors with ErrorOk()");
        //---------------------------------------------------------------

        TTB_CMD1 (so.CalcSync, 102);
        TTB_EXP  ("CallBack-Result: 204");

        TTB_INFO("ErrorCheck writes the complete error chain to TestEvents");

        TTB_CMD1(ErrorOk,17);
        TTB_EXP ("Error One");
        TTB_EXP ("  Error Two");
        TTB_EXP ("    Error Three");

        TTB_CMD1(ErrorOk,18);
        TTB_EXP ("Error Two");
        TTB_EXP ("  Error Three");

        TTB_CMD1(ErrorOk,19);
        TTB_EXP ("Error Three");

        TTB_CMD1(ErrorOk,20);
        TTB_EXP ("Error Four");

        TTB_CMD (ReportStoredErrors);
        TTB_EXP ("Error One");
        TTB_EXP ("  Error Two");
        TTB_EXP ("    Error Three");
        TTB_EXP ("Error Four");


        TTB_INFO("\nCalling so.CalcSyncAndReturnErrorCode(301)");
        if (ErrorOk(so.CalcSyncAndReturnErrorCode(301)))
        {
            TTB_EXP ("CallBack-Result: 602");
            TTB_INFO("Call succeeded!");
        }
        else
        {
            TTB_INFO("Call failed!");
        }
        TTB_EXP ("General calculation error");

        TTB_CMD (DeleteStoredErrors);


        TTB_CMD1(ErrorOk,so.CalcSyncAndReturnErrorCode(123));
        TTB_EXP ("CallBack-Result: 246");

        TTB_CMD1(ErrorOk,so.CalcSyncAndReturnErrorCode(302));
        TTB_EXP ("Wrong input value");
        TTB_EXP ("  Range exceeded");
        TTB_EXP ("    Value to big");
        TTB_EXP ("  Calculation cancelled");

        TTB_CMD (ReportStoredErrors);
        TTB_EXP ("Wrong input value");
        TTB_EXP ("  Range exceeded");
        TTB_EXP ("    Value to big");
        TTB_EXP ("  Calculation cancelled");


        //---------------------------------------------------------------
        TTB_SECTION("Optional check for catching exceptions within TTB)");
        //---------------------------------------------------------------

        if (TTB::TheEnvironment()->IsExistingCommandLineOption("-testOfExceptions"))
		{
			throw 47;
		}
	}
}

