// TestRunner.cpp : Definiert den Einstiegspunkt fr die Konsolenanwendung.

// some comment

#include "stdafx.h"

// simply include the implementation of the complete TestToolBox
#include <TestToolBox/TestToolBoxImpl.h>

// specific includes needed for accessing your test object
#include "./SourcesToTest/SimpleObject.h"
#include "CallbackFromSimpleObject.h"


#ifdef DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace std;
namespace TTB = TestToolBox;
using namespace TestToolBox::ShortNames;

// Define name of this component for simple trace utility and set initial trace level
TTB_TRC_DEF_GLOBALS ("TestRunner", TL_VAL_DEBUG);


//-----------------------------------------------------------------------------
// Optional: you can have some global/static data, possibly allocating some memory
//-----------------------------------------------------------------------------

// Simple example struct also allocating and releasing memory on the heap
struct SomeGlobalData
{
    SomeGlobalData()
    {
        TRCF("SomeGlobalData");
        TRC(TL_DEBUG, "Constructor");
        m_pSomeInt = new int;
    }

    ~SomeGlobalData()
    {
        TRCF("~SomeGlobalData");
        TRC(TL_DEBUG, "Destructor");
        // To test memory leak detection you can comment
        // the following line
        Reset();
    }

    // Function may also be called from within main
    void Reset (void)
    {
        delete m_pSomeInt;
        m_pSomeInt=0;
    }

    int* m_pSomeInt;
};
static SomeGlobalData g_someGlobalData;


//-----------------------------------------------------------------------------
// Optional: you can define specific test extensions to add "test fixtures" 
// for preparation and cleanup of test functions
//-----------------------------------------------------------------------------

// Simple example extension class mainly producing some output.
// All methods are optional. You can simplify your code by implementing only
// those methods you really need
class MyTestExtensions : public TestToolBox::RunnerExtensionPoints
{
public:

    // Constructor
    MyTestExtensions()
    {
        TRCF("MyTestExtensions");
        TRC(TL_DEBUG, "Constructor");

        // Here you can add code to initialize something within your test
        // environment. The constructor is executed before your test
        // functions are called
        // ... add some code for preparation

        // You can also write to test protocol
        TTB::Protocol::Get()->Writeln(
            TTB::OutputLevel::eVERBOUS,
            TTB::OutputMode::M_OUTFILE, 
            "MyTestExtensions::Constructor");

        // You can also define test cases here
        TTB_TEST_CASE("Test case defined within MyTestExtensions-Constructor");
        // ... code for checking something

        // General settings for your test environment
        TTB::TheGlobalOptions()->Set(Option::oOBJECT_NAME);
        TTB::TheGlobalOptions()->Set(Option::oSYNC);
    }

    // Destructor
    ~MyTestExtensions()
    {
        TRCF("~MyTestExtensions");
        TRC(TL_DEBUG, "Destructor");

        // Here you can add code to cleanup something you have used
        // within your test functions.
        // The destructor is executed after all your test
        // functions are called
        // ... add some code for cleanup

        // You still can write to test protocol
        TTB::Protocol::Get()->Writeln(
            TTB::OutputLevel::eVERBOUS,
            TTB::OutputMode::M_OUTFILE, 
            "MyTestExtensions::Destructor");

        // You still can define test cases here
        TTB_TEST_CASE("Test case defined within MyTestExtensions-Destructor");

        // explicitly free memory allocated outside of main function
        // alternatively perform memory leak check only at shutdown
        // via option "-checkForMemLeaks AT_SHUTDOWN"
        g_someGlobalData.Reset();
    }

    // Optional: overrides for empty base implementation of extension points

    // Typically you will implement here some initialization common for
    // all test functions. By evaluating the given FuncInfo you also can
    // choose your actions depending on the called function.
    virtual void BeforeCallOfTestFunction(TTB::FuncInfo const & in_info)
    {
        TRCF("MyTestExtensions::BeforeCallOfTestFunction");
        TRC(TL_DEBUG, "func=%s, file=%s, line=%d",
            in_info.m_funcName.c_str(),
            in_info.m_fileName.c_str(),
            in_info.m_lineNum
            );

        TTB::Protocol::Get()->Writeln(
            TTB::OutputLevel::eVERBOUS, TTB::OutputMode::M_OUTFILE, 
            "MyTestExtensions::BeforeCallOfTestFunction func=" + in_info.m_funcName
            + " file=" + in_info.m_fileName);
    }

    // Typically you will implement here some cleanup common for
    // all test functions. By evaluating the given FuncInfo you also can
    // choose your actions depending on the called function.
    virtual void AfterCallOfTestFunction(TTB::FuncInfo const & in_info)
    {
        TRCF("MyTestExtensions::AfterCallOfTestFunction");
        TRC(TL_DEBUG, "func=%s, file=%s, line=%d",
            in_info.m_funcName.c_str(),
            in_info.m_fileName.c_str(),
            in_info.m_lineNum
            );

        TTB::Protocol::Get()->Writeln(
            TTB::OutputLevel::eVERBOUS, TTB::OutputMode::M_OUTFILE, 
            "MyTestExtensions::AfterCallOfTestFunction func=" + in_info.m_funcName
            + " file=" + in_info.m_fileName);
    }

};


//-----------------------------------------------------------------------------
// Mandatory: Tell TestRunner how to execute your test functions
//-----------------------------------------------------------------------------

// automatic execution of main with execution of all test functions
// via TTB::TestRunner
// TTB_USE_DEFAULT_TEST_RUNNER();

// automatic execution of main with execution of all test functions
// via TTB::TestRunner but providing specific initializations and extension point
// implementations
TTB_USE_TEST_RUNNER_WITH_EXTENSIONS(MyTestExtensions);


//-----------------------------------------------------------------------------
// Mandatory: Define your test functions with your test cases
//-----------------------------------------------------------------------------

// The name of the test function is used as the description
TTB_TEST_FUNC (FirstTestFuncWithSimpleObject)
{
    // The name of the test function implicitly is the name of the first test case.

    // Instantiate the object to test
    SimpleObject so;

    // Instantiate a callback object and connect with the object to test
    CallBackFromSimpleObject cb(false);
    so.Connect(&cb);

    // Call an interface function of the object to test.
    // For this test object the result will synchronously be delivered over
    // the callback interface, which implicitly writes all received
    // calls to TestEvents and thus allows immediate checking of results.
    so.CalcSync(7);

    // Check the received result
    TTB_EXP("CallBack-Result: 14");
}


// You can use more extended descriptions for your test case
TTB_TEST_FUNC_DESC (SecondTestFunc,"Second test func - synchronous call to SimpleObject - no errors")
{
    // The description of the test function implicitly is the name of the first test case

    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb(false);
    so.Connect(&cb);

    so.CalcSync(12);
    TTB_EXP("CallBack-Result: 24");
}

TTB_TEST_FUNC (ThirdTestFunc)
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb(false);
    so.Connect(&cb);

    so.CalcSync(123);
    TTB_EXP("CallBack-Result: 246");

    // Here is an example of defining an additional test case within your test function

    TTB_TEST_CASE("ThirdTestFunc, next test case, big positive numbers");

    so.CalcSync(2020);
    TTB_EXP("CallBack-Result: 4040");
    so.CalcSync(3333);
    TTB_EXP("CallBack-Result: 6666");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (OptionsErrorNoObjectName,"SimulatedObjectBase-Option: oERROR, oNO_OBJECT_NAME")
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb(false);
    // we are not using synchronization within this test case
    cb._SetOption(Option::oSYNC, Option::ALL, OptionType::NOT_ACTIVE);
    so.Connect(&cb);

    //-------------------------------------------------------------------------
    TTB_SECTION("Using simulated call back regularly without error");
    //-------------------------------------------------------------------------

    so.CalcSync(-1);
    TTB_EXP("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-2);
    TTB_EXP("SpecialResultA in_val=-4 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-3);
    TTB_EXP("SpecialResultA in_val=-6 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-4);
    TTB_EXP("SpecialResultA in_val=-8 out_value=4711 (CbSimpleObject)");


    //-------------------------------------------------------------------------
    TTB_SECTION("Using options oERROR and oNO_OBJECT_NAME (PERMANENTLY_ACTIVE)");
    //-------------------------------------------------------------------------

    cb._SetOption(Option::oERROR);
    cb._SetOption(Option::oOBJECT_NAME, Option::ALL, OptionType::NOT_ACTIVE);
    so.CalcSync(-1);
    TTB_EXP("SpecialResultA in_val=-2 return error code =-1");
    so.CalcSync(-2);
    TTB_EXP("SpecialResultA in_val=-4 return error code =-1");
    so.CalcSync(-3);
    TTB_EXP("SpecialResultA in_val=-6 return error code =-1");
    so.CalcSync(-4);
    TTB_EXP("SpecialResultA in_val=-8 return error code =-1");


    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oERROR (AFTER_N_CALLS_PERMANENTLY_ACTIVE, N=3)");
    //-------------------------------------------------------------------------

    cb._ResetOptions();
    // we are not using synchronization within this test case
    cb._SetOption(Option::oSYNC, Option::ALL, OptionType::NOT_ACTIVE);
    cb._SetOption(Option::oERROR, "SpecialResultA", OptionType::AFTER_N_CALLS_PERMANENTLY_ACTIVE, 3);
    so.CalcSync(-1);
    TTB_EXP("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-2);
    TTB_EXP("SpecialResultA in_val=-4 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-3);
    TTB_EXP("SpecialResultA in_val=-6 return error code =-1 (CbSimpleObject)");
    so.CalcSync(-4);
    TTB_EXP("SpecialResultA in_val=-8 return error code =-1 (CbSimpleObject)");

    //-------------------------------------------------------------------------
    TTB_SECTION("Using options oERROR (AFTER_N_CALLS_ACTIVE_ONCE, N=2, scope=ALL)");
    //-------------------------------------------------------------------------
    cb._ResetOption(Option::oERROR, "SpecialResultA");
    // Setting global options is also possible
    TheGlobalOptions()->Set(Option::oERROR, "CallBackFromSimpleObject::SpecialResultA",OptionType::AFTER_N_CALLS_ACTIVE_ONCE, 2);
    so.CalcSync(-1);
    TTB_EXP("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-2);
    TTB_EXP("SpecialResultA in_val=-4 return error code =-1 (CbSimpleObject)");
    so.CalcSync(-3);
    TTB_EXP("SpecialResultA in_val=-6 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-4);
    TTB_EXP("SpecialResultA in_val=-8 out_value=4711 (CbSimpleObject)");
    TheGlobalOptions()->Reset(Option::oERROR);
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (OptionSilent,"SimulatedObjectBase-Option: oSILENT")
{
    // Prepare for testing
    SimpleObject so;

	CallBackFromSimpleObject cb(false);
    // we are not using synchronization within this test case
    cb._SetOption(Option::oSYNC, Option::ALL, OptionType::NOT_ACTIVE);
    so.Connect(&cb);

    //-------------------------------------------------------------------------
    TTB_SECTION("Using simulated call back regularly without oSILENT");
    //-------------------------------------------------------------------------

    so.CalcSync(-1);
    TTB_EXP("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-12);
    TTB_EXP("SpecialResultB in_val=-24 out_value=4712 (CbSimpleObject)");
    so.CalcSync(-23);
    TTB_EXP("SpecialResultC in_val=-46 out_value=4713 (CbSimpleObject)");


    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oSILENT (PERMANENTLY_ACTIVE, scope=SpecialResultB)");
    //-------------------------------------------------------------------------

    cb._SetOption(Option::oSILENT,"SpecialResultB");
    so.CalcSync(-1);
    TTB_EXP("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    so.CalcSync(-12);
    // No output expected!
    so.CalcSync(-23);
    TTB_EXP("SpecialResultC in_val=-46 out_value=4713 (CbSimpleObject)");


    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oSILENT (PERMANENTLY_ACTIVE, scope=ALL)");
    //-------------------------------------------------------------------------

    cb._SetOption(Option::oSILENT, Option::ALL);
    so.CalcSync(-1);
    // No output expected!
    so.CalcSync(-12);
    // No output expected!
    so.CalcSync(-23);
    // No output expected!
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (OptionNoSync,"SimulatedObjectBase-Option: oNO_SYNC")
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    //-------------------------------------------------------------------------
    TTB_SECTION("Using simulated call back regularly without oNO_SYNC");
    //-------------------------------------------------------------------------

    TTB_INIT_SYNC(1);
    so.CalcAsync(-1);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");

    TTB_INIT_SYNC(2);
    so.CalcSync(-12);
    so.CalcSync(-23);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultB in_val=-24 out_value=4712 (CbSimpleObject)");
    TTB_EXP("SpecialResultC in_val=-46 out_value=4713 (CbSimpleObject)");


    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oSYNC (NOT_ACTIVE, scope=SpecialResultA)");
    //-------------------------------------------------------------------------

    cb._ResetOptions();
    cb._SetOption(Option::oSYNC, "SpecialResultA", OptionType::NOT_ACTIVE);

    TTB_INFO("Only the last 2 calls will be synchronized.\n"
        "Important for testing: if there are further calls which are not synchronized they could not be checked securely.");
    TTB_INIT_SYNC(2);
    so.CalcAsync(-1);
    so.CalcAsync(-12);
    so.CalcAsync(-23);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultB in_val=-24 out_value=4712 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultC in_val=-46 out_value=4713 (CbSimpleObject)");


    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oSILENT implicitly includes option eNO_SYNC (PERMANENTLY_ACTIVE, scope=SpecialResultB)");
    //-------------------------------------------------------------------------

    cb._ResetOptions();
    TheGlobalOptions()->Set(Option::oSILENT,"SpecialResultB");

    TTB_INIT_SYNC(2);
    so.CalcAsync(-1);
    so.CalcAsync(-12);
    so.CalcAsync(-23);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultC in_val=-46 out_value=4713 (CbSimpleObject)");

	::Sleep(50); // ensure that all unsynchronized actions have terminated

    TheGlobalOptions()->Reset(Option::oSILENT,"SpecialResultB");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (OptionsBlocking,"SimulatedObjectBase-Option: oBLOCKING - temporarily changed output format")
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oBLOCKING (PERMANENTLY_ACTIVE, scope=SpecialResultA) ");
    //-------------------------------------------------------------------------

    cb._SetOption(Option::oBLOCKING, "SpecialResultA");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-1);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultA-Start in_val=-2 (CbSimpleObject)");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution,"SpecialResultA");
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultA-Stop out_value=4711 (CbSimpleObject)");

    TTB_INFO("\nSpecialResultB is NOT blocked");
    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-11);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultB in_val=-22 out_value=4712 (CbSimpleObject)");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-2);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultA-Start in_val=-4 (CbSimpleObject)");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution,"SpecialResultA");
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultA-Stop out_value=4711 (CbSimpleObject)");

    TTB_CMD2(cb._ResetOption,Option::oBLOCKING, "SpecialResultA");

    TTB_INIT_SYNC(2);
    TTB_CMD1(so.CalcAsync,-3);
    TTB_CMD1(so.CalcAsync,-4);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultA in_val=-6 out_value=4711 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultA in_val=-8 out_value=4711 (CbSimpleObject)");

    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oBLOCKING (AFTER_N_CALLS_ACTIVE_ONCE, N=2, scope=ALL) - changed output format ");
    //-------------------------------------------------------------------------

    namespace FI = TestToolBox::Formating;

    // Define format for output of method calls
    FI::FormatInstruction instruction;
    instruction.AddInfoBlock(FI::InfoBlock(Formating::CLASS_NAME,"","::"));
    instruction.AddInfoBlock(FI::InfoBlock(Formating::METHOD_NAME,""));
    instruction.AddInfoBlock(FI::InfoBlock(Formating::OBJECT_NAME," (", ")"));
    instruction.AddInfoBlock(FI::InfoBlock(Formating::USER_TEXT));
    TheFormatter()->SaveFormatInstructionForMethodCall();
    TheFormatter()->SetFormatInstructionForMethodCall(instruction);

    cb._SetOption(Option::oBLOCKING, Option::ALL, OptionType::AFTER_N_CALLS_ACTIVE_ONCE, 2);

    TTB_INIT_SYNC(3);
    TTB_CMD1(so.CalcAsync,-1);
    TTB_CMD1(so.CalcAsync,-11);
    TTB_CMD1(so.CalcAsync,-22);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("CallBackFromSimpleObject::SpecialResultA (CbSimpleObject) in_val=-2 out_value=4711");
    TTB_EXP_VAR("CallBackFromSimpleObject::SpecialResultB-Start (CbSimpleObject) in_val=-22");
    TTB_EXP_VAR("CallBackFromSimpleObject::SpecialResultC (CbSimpleObject) in_val=-44 out_value=4713");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution,"SpecialResultB");
    TTB_WAIT_SYNC();
    TTB_EXP("CallBackFromSimpleObject::SpecialResultB-Stop (CbSimpleObject) out_value=4712");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-11);
    TTB_WAIT_SYNC();
    TTB_EXP("CallBackFromSimpleObject::SpecialResultB (CbSimpleObject) in_val=-22 out_value=4712");

    TheFormatter()->RestoreFormatInstructionForMethodCall();

    //-------------------------------------------------------------------------
    TTB_SECTION("Using option oBLOCKING (AFTER_N_CALLS_ACTIVE_ONCE, N=2, scope=SpecialResultA), setting oERROR while blocking");
    //-------------------------------------------------------------------------

    cb._SetOption(Option::oBLOCKING, "SpecialResultA", OptionType::AFTER_N_CALLS_ACTIVE_ONCE, 2);

    TTB_INIT_SYNC(2);
    TTB_CMD1(so.CalcAsync,-1);
    TTB_CMD1(so.CalcAsync,-2);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultA in_val=-2 out_value=4711 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultA-Start in_val=-4 (CbSimpleObject)");

    TTB_CMD4(cb._SetOption,Option::oERROR, Option::ALL, OptionType::AFTER_N_CALLS_ACTIVE_ONCE, 1);

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution,"SpecialResultA");
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultA-Stop return error code =-1 (CbSimpleObject)");

    TTB_INIT_SYNC(2);
    TTB_CMD1(so.CalcAsync,-3);
    TTB_CMD1(so.CalcAsync,-4);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultA in_val=-6 out_value=4711 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultA in_val=-8 out_value=4711 (CbSimpleObject)");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (SimulatedObjectBase_TTB_METHOD_CALL,"SimulatedObjectBase-TTB_METHOD_CALL")
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    TTB_INFO("\nMethod name and object name is supported by base implementation");

    TTB_INIT_SYNC(2);
    TTB_CMD1(so.CalcAsync,-31);
    TTB_CMD1(so.CalcAsync,-32);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultD (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultD (CbSimpleObject)");

    TTB_INFO("\nBlocking and continuing a call is supported by base implementation");

    cb._SetOption(Option::oBLOCKING);
    cb._SetOption(Option::oOBJECT_NAME, Option::ALL, OptionType::NOT_ACTIVE);

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-33);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultD-Start");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution, "SpecialResultD");
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultD-Stop");
}

//-----------------------------------------------------------------------------

TTB_TEST_FUNC_DESC (SimulatedObjectBase_TTB_METHOD_CALL1,"SimulatedObjectBase-TTB_METHOD_CALL1")
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    TTB_INFO("\nMethod name and object name is supported by base implementation");

    TTB_INIT_SYNC(2);
    TTB_CMD1(so.CalcAsync,-41);
    TTB_CMD1(so.CalcAsync,-42);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultE in_value=-82 out_value=4715 (CbSimpleObject)");
    TTB_EXP_VAR("SpecialResultE in_value=-84 out_value=4715 (CbSimpleObject)");

    TTB_INFO("\nBlocking and continuing a call is supported by base implementation");

    cb._SetOption(Option::oBLOCKING);
    cb._SetOption(Option::oOBJECT_NAME, Option::ALL, OptionType::NOT_ACTIVE);

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-43);
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultE-Start in_value=-86 out_value=4715");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution, "SpecialResultE");
    TTB_WAIT_SYNC();
    TTB_EXP_VAR("SpecialResultE-Stop");

    cb._ResetOption(Option::oBLOCKING);
}

//-----------------------------------------------------------------------------

CallBackFromSimpleObject* FindMyObject(std::string const & in_obhjectName)
{
    return static_cast<CallBackFromSimpleObject*>(SimObject(in_obhjectName));
}

TTB_TEST_FUNC_DESC (SimulatedObjectBase_RetMethodCallA,"SimulatedObjectBase-RetMethodCall-A")
{
    // Prepare for testing
    SimpleObject so;
    CallBackFromSimpleObject cb;
    so.Connect(&cb);

    TTB_INFO("\nMethod name and object name is supported by base implementation");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-55);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultF in_val=-110 (CbSimpleObject)");

    // Use global registration to access callback object
    SimObject("CbSimpleObject")->_SetOption(Option::oERROR, "SpecialResultF");
    // Call a method of your specific class
    FindMyObject("CbSimpleObject")->Result(12);
    TTB_EXP("CallBack-Result: 12");
    //TheRegisteredSimulatedObjects()->Get("CbSimpleObject")->_SetOption(Option::oERROR, "SpecialResultF");
    // cb._SetOption(Option::oERROR, "SpecialResultF");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-56);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultF in_val=-112 return error (CbSimpleObject)");

    cb._ResetOption(Option::oERROR, "SpecialResultF");
    cb._SetOption(Option::oBLOCKING, "SpecialResultF");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-57);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultF-Start in_val=-114 (CbSimpleObject)");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution, "SpecialResultF");
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultF-Stop (CbSimpleObject)");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-58);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultF-Start in_val=-116 (CbSimpleObject)");

    // While halting the return value can be set
    cb._SetOption(Option::oERROR, "SpecialResultF");

    TTB_INIT_SYNC(1);
    TTB_CMD1(cb._ContinueExecution, "SpecialResultF");
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultF-Stop return error (CbSimpleObject)");

    TTB_INIT_SYNC(1);
    TTB_CMD1(so.CalcAsync,-75);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultH (CbSimpleObject)");

    cb._SetOption(Option::oERROR, "SpecialResultH");

    TTB_CMD1(so.CalcAsync,-75);
    TTB_WAIT_SYNC();
    TTB_EXP("SpecialResultH return error code -1 (CbSimpleObject)");

    SomeErrorResult err;
    long ignoreVal;
    TTB_CMD2(err=cb.SpecialResultH,5,ignoreVal);
    TTB_EXP("SpecialResultH return error code -1 (CbSimpleObject)");
    TTB_CHECK_EQUAL(err.m_errorCode,-1);
    TTB_CHECK_EQUAL(err.m_errorInfo,std::string("Simulated error in CallBackFromSimpleObject::SpecialResultH"));

	// Demo: using aCCESS TO CMD LINE PARAMS OR HOW TO DEFINE ARBITRARY VALUES

	int counter = TTB::TheEnvironment()->GetCommandLineOptionVal<int>("-SomeCounter");
    TTB_CHECK_EQUAL(counter,123);

	TTB::TheEnvironment()->SetCommandLineOptionVal("-SomeCounter", 4711);
	counter = TTB::TheEnvironment()->GetCommandLineOptionVal<int>("-SomeCounter");
    TTB_CHECK_EQUAL(counter,4711);
}


//-----------------------------------------------------------------------------

namespace TestToolBox
{
    std::string MakeReturnValue(double& out_retVal, bool in_simulateFailure, std::string const & in_methodName, SimulatedObjectBase const * in_pObject)
    {
        if (in_simulateFailure) out_retVal = -99.99;
        return "";
    };

    std::string MakeReturnValue(int& out_retVal, bool in_simulateFailure, std::string const & in_methodName, SimulatedObjectBase const * in_pObject)
    {
        if (in_simulateFailure) out_retVal = -1;
        return ""; // use default error msg from SimulatedObjectBase
    };

    std::string MakeReturnValue(SomeErrorResult& out_retVal, bool in_simulateFailure, std::string const & in_methodName, SimulatedObjectBase const * in_pObject)
    {

        if (in_simulateFailure)
        {
            out_retVal.m_errorCode = -1;
            out_retVal.m_errorInfo = "Simulated error in " + in_pObject->GetClassName() + "::" + in_methodName;
        }
        else // if not yet done within default construction
        {
            out_retVal.m_errorCode = 0;
            out_retVal.m_errorInfo = "";
        }

        // to override the default error info "return error"
        std::string info;
        if (in_simulateFailure)
        {
            std::ostringstream oss;
            oss << "return error code " << out_retVal.m_errorCode;
            info = oss.str();
        }
        return info;
    };

}
