////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of ExampleDLL
/// 
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ExampleDll.h"
#include <iostream>

#include "TestToolBox\EnvironmentImpl.h"
#include "TestToolBox\TestEventsImpl.h"
#include "TestToolBox\SynchronizerImpl.h"
#include "TestToolBox\ProtocolImpl.h"
#include "TestToolBox\TestToolBoxImpl.h"

/// \cond

namespace TTB = TestToolBox;
using namespace std;

// Define name of this component for simple trace utility and set initial trace level
TTB_TRC_DEF_GLOBALS ("ExampleDll", TL_VAL_DEBUG);


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif


//*****************************************************************************
//*****************************************************************************


void AccessEnvironmentWithinExampleDll (void)
{
    std::cout << "AccessEnvironmentWithinExampleDll-Begin" << std::endl;
    TRCF("AccessEnvironmentWithinExampleDll");
    TRC(TL_DEBUG,"Begin");


    //----- Access to TestToolBox::Environment -----

    TTB::Environment* pEnvironment = TTB::TheEnvironment();

    cout << "Environment="  << pEnvironment << endl;
    cout << "GetSourceDir=" << pEnvironment->GetSourceDir() << endl;  
    cout << "GetOutputDir=" << pEnvironment->GetOutputDir() << endl;

    cout << "Change path settings of environment" << endl;
    pEnvironment->SetSourceDir("Q:/SomePathSetFromExampleDll");
    pEnvironment->SetOutputDir("R:/AnotherPathSetFromExampleDll");

    TRC(TL_DEBUG,"End");
    std::cout << "AccessEnvironmentWithinExampleDll-End" << std::endl;

} // AccessEnvironmentWithinExampleDll


//*****************************************************************************
//*****************************************************************************


void AccessTestEventsWithinExampleDll (void)
{
    std::cout << "AccessTestEventsWithinExampleDll-Begin" << std::endl;
    TRCF("AccessTestEventsWithinExampleDll");
    TRC(TL_DEBUG,"Begin");

    //----- Access to TestToolBox::TestEvents -----

    TTB::TestEvents* pEvents = TTB::TestEvents::Get();
    pEvents->Act("Also in Dll an event occured");
    pEvents->ListAllStoredEvents();


    //----- Access to TestToolBox::GlobalOptions -----

    bool optionIsSet = TTB::TheGlobalOptions()->IsSet("SomeOptionForDll");
    pEvents->Act("SomeOptionForDll is set: " + TTB::ToString(optionIsSet));


    //----- Access to TestToolBox::Synchronizer -----

    pEvents->Act("Dll will now rise a sync event...");
    TTB_SYNC();

    TTB::Protocol::Get()->Writeln(TTB::OutputLevel::eREGULAR,TTB::OutputMode::M_STDOUT,
        "Message only for StdOut");

    TTB::Protocol::Get()->Writeln(TTB::OutputLevel::eREGULAR,TTB::OutputMode::M_STDOUT|TTB::OutputMode::M_RPTFILE,
        "This message is directed only to RPT file and stdout");

    TRC(TL_DEBUG,"End");
    std::cout << "AccessTestEventsWithinExampleDll-End" << std::endl;

} // AccessEnvironmentWithinExampleDll


//*****************************************************************************
//*****************************************************************************

/// \endcond
