#include "stdafx.h"

#include "MyTestCommands.h"
#include <pugixml.hpp>

#include <TestToolBox\TestToolBox.h>
#include <boost/test/unit_test.hpp>
namespace TTB = TestToolBox;

//-----------------------------------------------------------------------------

class MyTestSuiteFixture
{
public:
	MyTestSuiteFixture()
	{
		TTB_INFO("\nMyTestSuiteFixture-Constructor");

	}
	~MyTestSuiteFixture()
	{
		TTB_INFO("\nMyTestSuiteFixture-Destructor");
		// Reset XmlCheck to default
		TTB::TheXmlCheck()->SetDefaults();
	}
};

//-----------------------------------------------------------------------------

BOOST_FIXTURE_TEST_SUITE(SuiteApplyFilter, MyTestSuiteFixture)

//-----------------------------------------------------------------------------

TTB_BOOST_TEST_CASE(GenerateFilteredXmlTree)
{
	pugi::xml_document doc;
	pugi::xml_node root = TTB::TheXmlCheck()->ReadFromFile(doc, MY_XML_FILE_PATH);

	// Add some pointer values to existing XML tree
	pugi::xml_node nodePointers = root.append_child("NodeWithPointerValues");
	nodePointers.append_attribute("pSomething") = "0x2134683";
	nodePointers.append_attribute("pNothing") = "00000000";
	pugi::xml_node nodePointerNodeA = nodePointers.append_child("PointsToSomethingElse");
	nodePointerNodeA.append_child(pugi::node_pcdata).set_value("0x24315683");
	pugi::xml_node nodePointerNodeB = nodePointers.append_child("PointsToNothing");
	nodePointerNodeB.append_child(pugi::node_pcdata).set_value("0");

	//---------------------------------------------------------------
	TTB_SECTION("Unfiltered check of xml tree");
	//---------------------------------------------------------------

	TTB::TheXmlCheck()->CheckNode(doc);
	TTB_EXP("MyContainer");
	TTB_EXP("  Group_A");
	TTB_EXP("    Vector");
	TTB_EXP("      id: 1");
	TTB_EXP("      date: 01.07.09");
	TTB_EXP("      time: 09:15:12.234");
	TTB_EXP("      tolerance: 0.0258");
	TTB_EXP("      This is the text of the first vector element");
	TTB_EXP("      x");
	TTB_EXP("        3.14235");
	TTB_EXP("    Vector");
	TTB_EXP("      id: 2");
	TTB_EXP("      date: 01.07.09");
	TTB_EXP("      time: 09:15:12.234");
	TTB_EXP("      tolerance: 0.0375451");
	TTB_EXP("      This is the text of the second vector element");
	TTB_EXP("      x");
	TTB_EXP("        3.14235");
	TTB_EXP("    Vector");
	TTB_EXP("      id: 3");
	TTB_EXP("      tolerance: 1.357");
	TTB_EXP("      x");
	TTB_EXP("        6.28532");
	TTB_EXP("  Group_B");
	TTB_EXP("    x");
	TTB_EXP("      1.23456");
	TTB_EXP("    x");
	TTB_EXP("      2.3581");
	TTB_EXP("    x");
	TTB_EXP("      3.132437");
	TTB_EXP("    x");
	TTB_EXP("      4.987654");
	TTB_EXP("    x");
	TTB_EXP("      5.29741");
	TTB_EXP("  Group_C");
	TTB_EXP("    Interior");
	TTB_EXP("      Something");
	TTB_EXP("        Some text");
	TTB_EXP("      date");
	TTB_EXP("        28.09.09");
	TTB_EXP("  NodeWithPointerValues");
	TTB_EXP("    pSomething: 0x2134683");
	TTB_EXP("    pNothing: 00000000");
	TTB_EXP("    PointsToSomethingElse");
	TTB_EXP("      0x24315683");
	TTB_EXP("    PointsToNothing");
	TTB_EXP("      0");


	//---------------------------------------------------------------
	TTB_SECTION("Apply filter to xml tree and repeat unfiltered check");
	//---------------------------------------------------------------

	TTB::TheXmlCheck()->IgnoreNode("Group_B");
	TTB::TheXmlCheck()->IgnoreNode("Group_C");
	TTB::TheXmlCheck()->IgnoreAttribute("date");
	TTB::TheXmlCheck()->IgnoreAttribute("time");
	TTB::TheXmlCheck()->UseAsDoubleAttribute("tolerance", true, 2);
	TTB::TheXmlCheck()->UseAsDoubleNode("x", true, 3);
	TTB::TheXmlCheck()->UseAsPointerAttribute("pSomething");
	TTB::TheXmlCheck()->UseAsPointerAttribute("pNothing");

    /// [xmlcheck apply filter]
    // Define some filter
    TTB::TheXmlCheck()->IgnoreNode("Group_C");
    TTB::TheXmlCheck()->IgnoreAttribute("date");
    TTB::TheXmlCheck()->UseAsPointerNode("PointsToSomethingElse");
	TTB::TheXmlCheck()->UseAsPointerNode("PointsToNothing");
    //...
    // Change the contents of given XML document / tree according to
    // current filter settings:
    //  - all attributes or nodes to ignore are removed from the tree
    //  - double values are adjusted to desired precision
    //  - pointer values are changed to "0" or "exists" 
	TTB::TheXmlCheck()->ApplyFilterToXmlTree(doc);

    // Save the reduced tree to some logfile
	const std::string MY_XML_FILE_PATH_FILTERED = MY_TEST_DIR + "/FilteredPugiXmlFile.xml";
	TTB::TheXmlCheck()->SaveToFile(doc, MY_XML_FILE_PATH_FILTERED);
    /// [xmlcheck apply filter]

    /// [xmlcheck dump with verification]
    TTB_INFO("Dump current settings and verify as test events");

    TTB::TheXmlCheck()->Dump(TTB::CTX_TEST_EVENT);
    TTB_EXP("m_outputMode: 256");
    TTB_EXP("m_detailedOutput: 0");
    TTB_EXP("m_attributesToIgnore:");
    TTB_EXP("  date");
    TTB_EXP("  time");
    TTB_EXP("m_nodesToIgnore:");
    TTB_EXP("  Group_B");
    TTB_EXP("  Group_C");
    TTB_EXP("m_doubleAttributes with precision:");
    TTB_EXP("  tolerance: 2");
    TTB_EXP("m_doubleNodes with precision:");
    TTB_EXP("  x: 3");
    TTB_EXP("m_pointerAttributes:");
    TTB_EXP("  pNothing");
    TTB_EXP("  pSomething");
    TTB_EXP("m_pointerNodes:");
    TTB_EXP("  PointsToNothing");
    TTB_EXP("  PointsToSomethingElse");
    /// [xmlcheck dump with verification]

    TTB::TheXmlCheck()->SetDefaults();

    /// [xmlcheck dump without verification]
    TTB_INFO("Dump current settings as info message to test protocol");

    TTB::TheXmlCheck()->Dump(TTB::CTX_INFO);
    // no test verification is needed
    /// [xmlcheck dump without verification]

    TTB::TheXmlCheck()->CheckNode(doc);
	TTB_EXP("MyContainer");
	TTB_EXP("  Group_A");
	TTB_EXP("    Vector");
	TTB_EXP("      id: 1");
	TTB_EXP("      tolerance: 0.03");
	TTB_EXP("      This is the text of the first vector element");
	TTB_EXP("      x");
	TTB_EXP("        3.142");
	TTB_EXP("    Vector");
	TTB_EXP("      id: 2");
	TTB_EXP("      tolerance: 0.04");
	TTB_EXP("      This is the text of the second vector element");
	TTB_EXP("      x");
	TTB_EXP("        3.142");
	TTB_EXP("    Vector");
	TTB_EXP("      id: 3");
	TTB_EXP("      tolerance: 1.36");
	TTB_EXP("      x");
	TTB_EXP("        6.285");
	TTB_EXP("  NodeWithPointerValues");
	TTB_EXP("    pSomething: exists");
	TTB_EXP("    pNothing: 0");
	TTB_EXP("    PointsToSomethingElse");
	TTB_EXP("      exists");
	TTB_EXP("    PointsToNothing");
	TTB_EXP("      0");

	//---------------------------------------------------------------
	TTB_SECTION("Read saved xml file and repeat unfiltered check");
	//---------------------------------------------------------------

	{
		pugi::xml_document doc;
		TTB::TheXmlCheck()->ReadFromFile(doc, MY_XML_FILE_PATH_FILTERED);
		TTB::TheXmlCheck()->CheckNode(doc);
		TTB_EXP("MyContainer");
		TTB_EXP("  Group_A");
		TTB_EXP("    Vector");
		TTB_EXP("      id: 1");
		TTB_EXP("      tolerance: 0.03");
		TTB_EXP("      This is the text of the first vector element");
		TTB_EXP("      x");
		TTB_EXP("        3.142");
		TTB_EXP("    Vector");
		TTB_EXP("      id: 2");
		TTB_EXP("      tolerance: 0.04");
		TTB_EXP("      This is the text of the second vector element");
		TTB_EXP("      x");
		TTB_EXP("        3.142");
		TTB_EXP("    Vector");
		TTB_EXP("      id: 3");
		TTB_EXP("      tolerance: 1.36");
		TTB_EXP("      x");
		TTB_EXP("        6.285");
		TTB_EXP("  NodeWithPointerValues");
		TTB_EXP("    pSomething: exists");
		TTB_EXP("    pNothing: 0");
		TTB_EXP("    PointsToSomethingElse");
		TTB_EXP("      exists");
		TTB_EXP("    PointsToNothing");
		TTB_EXP("      0");
	}
}}

//-----------------------------------------------------------------------------

TTB_BOOST_TEST_CASE(TestAutoRestoreXmlCheckAndLocalInstance)
{
	pugi::xml_document doc;
	pugi::xml_node root = TTB::TheXmlCheck()->ReadFromFile(doc, MY_XML_FILE_PATH);

	TTB::TheXmlCheck()->IgnoreNode("Group_B");
	TTB::TheXmlCheck()->IgnoreNode("Group_C");
	TTB::TheXmlCheck()->IgnoreAttribute("date");
	TTB::TheXmlCheck()->IgnoreAttribute("time");
	TTB::TheXmlCheck()->UseAsDoubleAttribute("tolerance", true, 2);
	TTB::TheXmlCheck()->UseAsDoubleNode("x", true, 3);
	TTB::TheXmlCheck()->UseAsPointerAttribute("pSomething");
	TTB::TheXmlCheck()->UseAsPointerAttribute("pNothing");
	TTB::TheXmlCheck()->UseAsPointerNode("PointsToSomethingElse");
	TTB::TheXmlCheck()->UseAsPointerNode("PointsToNothing");

    //---------------------------------------------------------------
    TTB_SECTION("Check with active filter conditions");
    //---------------------------------------------------------------

	TTB::TheXmlCheck()->CheckNode(doc);
    TTB_EXP("MyContainer");
    TTB_EXP("  Group_A");
    TTB_EXP("    Vector");
    TTB_EXP("      id: 1");
    TTB_EXP("      tolerance: 0.03");
    TTB_EXP("      This is the text of the first vector element");
    TTB_EXP("      x");
    TTB_EXP("        3.142");
    TTB_EXP("    Vector");
    TTB_EXP("      id: 2");
    TTB_EXP("      tolerance: 0.04");
    TTB_EXP("      This is the text of the second vector element");
    TTB_EXP("      x");
    TTB_EXP("        3.142");
    TTB_EXP("    Vector");
    TTB_EXP("      id: 3");
    TTB_EXP("      tolerance: 1.36");
    TTB_EXP("      x");
    TTB_EXP("        6.285");

    //---------------------------------------------------------------
    TTB_SECTION("Check with  new filter conditions");
    //---------------------------------------------------------------

    /// [xmlcheck AutoRestoreXmlCheck]
    //...
    {
        // Store current settings
		TTB::AutoRestoreXmlCheck autoRestoreXmlCheck;
        // Clear all settings
		TTB::TheXmlCheck()->SetDefaults();
        // Define new settings
        TTB::TheXmlCheck()->IgnoreNode("Group_A");
        TTB::TheXmlCheck()->IgnoreNode("Group_B");

        // Do something with the new settings
        TTB::TheXmlCheck()->CheckNode(doc);
        TTB_EXP("MyContainer");
        TTB_EXP("  Group_C");
        TTB_EXP("    Interior");
        TTB_EXP("      Something");
        TTB_EXP("        Some text");
        TTB_EXP("      date");
        TTB_EXP("        28.09.09");

    } // previous settings for XmlCheck are restored when leaving scope
    //...
    /// [xmlcheck AutoRestoreXmlCheck]

    //---------------------------------------------------------------
    TTB_SECTION("Check with restored filter conditions");
    //---------------------------------------------------------------
    
    TTB::TheXmlCheck()->CheckNode(doc);
    TTB_EXP("MyContainer");
    TTB_EXP("  Group_A");
    TTB_EXP("    Vector");
    TTB_EXP("      id: 1");
    TTB_EXP("      tolerance: 0.03");
    TTB_EXP("      This is the text of the first vector element");
    TTB_EXP("      x");
    TTB_EXP("        3.142");
    TTB_EXP("    Vector");
    TTB_EXP("      id: 2");
    TTB_EXP("      tolerance: 0.04");
    TTB_EXP("      This is the text of the second vector element");
    TTB_EXP("      x");
    TTB_EXP("        3.142");
    TTB_EXP("    Vector");
    TTB_EXP("      id: 3");
    TTB_EXP("      tolerance: 1.36");
    TTB_EXP("      x");
    TTB_EXP("        6.285");

    //---------------------------------------------------------------
    TTB_SECTION("Using local instance of XmlCheck");
    //---------------------------------------------------------------

    /// [xmlcheck local instance]
    {
        // Create local instance of XmlCheck and connect it to TestEvents
        TTB::XmlCheckSp spMyXmlCheck = TTB::XmlCheck::CreateLocalInstance(
            TTB::TheTestEvents());

        // Define desired settings
        spMyXmlCheck->IgnoreNode("Group_A");
        spMyXmlCheck->IgnoreNode("Group_B");
        // Remark: settings of singleton instance of XmlCheck are not affected

        // Perform check
        spMyXmlCheck->CheckNode(doc);
        TTB_EXP("MyContainer");
        TTB_EXP("  Group_C");
        TTB_EXP("    Interior");
        TTB_EXP("      Something");
        TTB_EXP("        Some text");
        TTB_EXP("      date");
        TTB_EXP("        28.09.09");

    }  // when leaving scope local instance of XmlCheck is automatically destroyed
    /// [xmlcheck local instance]
}}

//-----------------------------------------------------------------------------

BOOST_AUTO_TEST_SUITE_END()
