#include "stdafx.h"

#include "MyTestCommands.h"
#include <pugixml.hpp>
#include <TestToolBox\TestToolBox.h>


#include <boost/test/unit_test.hpp>

namespace TTB = TestToolBox;

// This is an example of a test case file without any test suite

TTB_BOOST_TEST_CASE(Compare)
{
	/// [xmlcheck compare sample]
	static const std::string SHORT_DEMO_A =
		R"(<SomeNode id="3" attrA="1.2">
		<SubNodeB>0.2</SubNodeB>
	  </SomeNode>
	)";

	static const std::string SHORT_DEMO_B =
		R"(<SomeNode id="3" attrA="1.8">
		<SubNodeB>0.4</SubNodeB>
	  </SomeNode>
	)";
	/// [xmlcheck compare sample]

	static const std::string SHORT_DEMO_C =
		R"(<SomeNode id="3" attrA="1.8">
		<SubNodeB>0.4</SubNodeB>
        <UnexpectedNode>some data</UnexpectedNode>
	  </SomeNode>
	)";

	/// [xmlcheck compare read sample data]
	pugi::xml_document docA;
	pugi::xml_node rootA = TTB::TheXmlCheck()->ReadFromString(docA, SHORT_DEMO_A);
	pugi::xml_document docB;
	pugi::xml_node rootB = TTB::TheXmlCheck()->ReadFromString(docB, SHORT_DEMO_B);
	/// [xmlcheck compare read sample data]
	pugi::xml_document docC;
	pugi::xml_node rootC = TTB::TheXmlCheck()->ReadFromString(docC, SHORT_DEMO_C);

	/// [xmlcheck compare full]
	TTB_INFO("Compare A and B, check all entries of xml trees");
	bool equal = TTB::TheXmlCheck()->Compare(docA, docB);
	TTB_EXP("Difference detected");
	TTB_EXP("lhs: attrA: 1.2(path: /SomeNode/attrA, pos: 1)");
	TTB_EXP("rhs: attrA: 1.8(path: /SomeNode/attrA, pos: 1)");

	// Optional check of return value:
	TTB_CHECK_EQUAL(equal, false); // i.e. difference found
	/// [xmlcheck compare full]

	// Optional check of compare result string (useful when XmlCheck is not connected to test events)
	std::string compareResult = TTB::TheXmlCheck()->GetCompareResult();
	TTB_ACT_S(compareResult);
	TTB_EXP("Difference detected\n"
	        "lhs: attrA: 1.2(path: /SomeNode/attrA, pos: 1)\n"
			"rhs: attrA: 1.8(path: /SomeNode/attrA, pos: 1)");

	/// [xmlcheck compare ignore attribute]
	TTB_INFO("\nCompare A and B, ignore attribute attrA");
	TTB::TheXmlCheck()->IgnoreAttribute("attrA");
	equal = TTB::TheXmlCheck()->Compare(docA, docB);
	TTB_CHECK_EQUAL(equal, false); // i.e. difference found
	TTB_EXP("Difference detected");
	TTB_EXP("lhs: 0.2(path: /SomeNode/SubNodeB/, pos: 42)");
	TTB_EXP("rhs: 0.4(path: /SomeNode/SubNodeB/, pos: 42)");
	/// [xmlcheck compare ignore attribute]

	/// [xmlcheck compare ignore attribute and node]
	TTB_INFO("\nCompare A and B, ignore attribute attrA and node SubNodeB");
	TTB::TheXmlCheck()->IgnoreNode("SubNodeB");
	equal = TTB::TheXmlCheck()->Compare(docA, docB);
	TTB_CHECK_EQUAL(equal, true); // i.e. no difference found
	TTB_EXP("XML-Compare: no differences found");
	/// [xmlcheck compare ignore attribute and node]

	TTB_INFO("\nCompare A and C, ignore attribute attrA and node SubNodeB");
	equal = TTB::TheXmlCheck()->Compare(docA, docC);
	TTB_CHECK_EQUAL(equal, false);
	TTB_EXP("Difference detected, rhs has more data");
	TTB_EXP("rhs: UnexpectedNode (path: /SomeNode/UnexpectedNode, pos: 66)");

	//-------------------------------------------------------------------------
}}