// File: TestXmlCheckWithPugiXml.cpp

#include "stdafx.h"

//-----------------------------------------------------------------------------
// Configure usage of boost test
//-----------------------------------------------------------------------------

/// [boost.test configure]
// To write boost test cases within multiple source files
// define the name of your test module within exactly one cpp file
// and before including any boost test header.
// Do not define BOOST_TEST_MAIN. Otherwise you may get link errors
// like "init_unit_test_suite already defined"
#define BOOST_TEST_MODULE "Testing XmlCheck with PugiXml"

#include <boost/bind.hpp>
#include <boost/test/auto_unit_test.hpp>

// Use the boost auto link feature which automatically chooses the correct library version
#define BOOST_LIB_NAME boost_unit_test_framework
#include <boost/config/auto_link.hpp>
/// [boost.test configure]


//-----------------------------------------------------------------------------
// Add implementaion code of TestToolBox
//-----------------------------------------------------------------------------

/// [ttb include impl]
// directly include implementation code,
// possible configuration via preprocessor definitions within project settings
#include "TestToolBox\TestToolBoxImpl.h"

// Define name of this component for simple trace utility and set initial trace level
TTB_TRC_DEF_GLOBALS("TestXmlCheckWithPugiXmlc", TL_VAL_DEBUG);

namespace TTB = TestToolBox;
/// [ttb include impl]

//-----------------------------------------------------------------------------
// Other includes
//-----------------------------------------------------------------------------

#include "MyTestCommands.h"
#include <assert.h>
#include <iostream>
using namespace std;

//-----------------------------------------------------------------------------
// Global fixture
//-----------------------------------------------------------------------------

// Forward declaration
void InitialPreparation();

/// [boost.test global fixture impl]
// Global Fixture for onetime initialization and tear down.
// Base class DefaultSettingsForBoostAndTTB cares for proper initialization
// and cleanup of TestToolBox (e.g. test protocol files, recorded TestEvents)
struct MyGlobalSettingsForTestToolBox : public TTB::DefaultSettingsForBoostAndTTB
{
	MyGlobalSettingsForTestToolBox(void)
	{
		// In case of using TTB::TestEvents you can specify your desired
		// syntax for writing test events to protocol file.
		// This may be useful when writing / correcting test cases because
		// you can directly copy recorded test sequences from protocol file
		// to your source code.
		// When defining via command line the needed check macro as prefix
		// (e.g. "-prefixEvents TTB_EXP") you get the ready to copy format
		// in C++ syntax (TTB_EXP("Some event generated");).
        // If prefix string is not set the simple and more readable prefix
		// ">" is used.
		std::string prefixForTestEventsWithinOutFile =
			TTB::TheEnvironment()->GetCommandLineOption("-prefixEvents");
		bool detailedLog = true;
		TTB::TestEvents::Get()->SetDetailedLog(
			detailedLog, prefixForTestEventsWithinOutFile);

		// Specific preparations
		// e.g. generate test data, startup system under test,
		// execute initial test cases to be performed directly after startup
		InitialPreparation();
	}

	// Define your specific destructor if you have to perform special cleanup
	// at end of testing (e.g. shutdown test environment)
};
/// [boost.test global fixture impl]

/// [boost.test global fixture install]
// Install your specific class MyGlobalSettingsForTestToolBox
// as a global fixture for boost
BOOST_GLOBAL_FIXTURE(MyGlobalSettingsForTestToolBox);
/// [boost.test global fixture install]

//-----------------------------------------------------------------------------

// Regular boost test case not visible within TestToolBox
BOOST_AUTO_TEST_CASE(EmptyTestcaseToShowOriginalBoostTestCase)
{
	// check something here
}

//-----------------------------------------------------------------------------
