////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Implementation of class TestTime
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CommonSources/CommonDefinitions.h"
#include "TestTime.h"
#include <time.h>
#include <iomanip>

///\cond do not document


namespace TestToolBox
{

//************************************************************************
//************************************************************************


TTB_INLINE TestTime::TestTime():
    m_startTimeMs      (0),
    m_startTimeDateStr ("unknown"),
    m_stopTimeDateStr  ("unknown"),
    m_testDurationStr  ("unknown")
{
    StoreStartTime();

} // TestTime::TestTime


//************************************************************************
//************************************************************************


TTB_INLINE void TestTime::StoreStartTime()
{
    // Store tick count
    m_startTimeMs = ::GetTickCount();

    // Store time and date
    time_t timeStartOfTest = time(0);
    struct tm localTimeStartOfTest;
    localtime_s(&localTimeStartOfTest,&timeStartOfTest);
    m_startTimeDateStr = FormatTimeAndDate (&localTimeStartOfTest);

} // TestTime::StoreStartTime


//************************************************************************
//************************************************************************


TTB_INLINE void TestTime::StoreStopTime()
{
    // Calculate test duration
    long stopTimeMs = ::GetTickCount();
    m_testDurationStr = FormatTestDuration (stopTimeMs - m_startTimeMs);

    // Store time and date
    time_t timeStopOfTest = time(0);
    struct tm localTimeStopOfTest;
    localtime_s(&localTimeStopOfTest,&timeStopOfTest);
    m_stopTimeDateStr = FormatTimeAndDate (&localTimeStopOfTest);

} // TestTime::StoreStopTime


//************************************************************************
//************************************************************************


TTB_INLINE void TestTime::ResetStopTime()
{
    m_stopTimeDateStr = "unknown";
    m_testDurationStr = "unknown";
}

//************************************************************************
//************************************************************************


TTB_INLINE std::string TestTime::GetStartTimeDateStr (void)
    {return m_startTimeDateStr;}

TTB_INLINE std::string TestTime::GetStopTimeDateStr (void)
{
    if (m_testDurationStr == "unknown")
    {
        StoreStopTime();
    }
    return m_stopTimeDateStr;
}

TTB_INLINE std::string TestTime::GetCurTimeDateStr(void)
{
    // Store time and date
    time_t timeNow = time(0);
    struct tm localTimeNow;
    localtime_s(&localTimeNow,&timeNow);
    return FormatTimeAndDate (&localTimeNow);

} // TestTime::GetCurTimeDateStr

TTB_INLINE std::string TestTime::GetTestDurationStr (void)
{
    if (m_stopTimeDateStr == "unknown")
    {
        StoreStopTime();
    }
    return m_testDurationStr;
}

TTB_INLINE long TestTime::GetCurTimeMs (void)
    {return ::GetTickCount() - m_startTimeMs;}

TTB_INLINE long TestTime::GetCurTimeSec (void)
    {return GetCurTimeMs() / 1000;}

TTB_INLINE std::string TestTime::GetCurTimeStr (void)
    {return FormatTestDuration (::GetTickCount() - m_startTimeMs);}


//************************************************************************
//************************************************************************


TTB_INLINE std::string TestTime::FormatTimeAndDate (
    struct tm* in_pTime)
{
    std::ostringstream tmpStr;
    tmpStr << std::setw(2) << std::setfill('0') << in_pTime-> tm_mday << ".";
    tmpStr << std::setw(2) << std::setfill('0') << in_pTime-> tm_mon + 1 << ".";
    tmpStr << in_pTime-> tm_year + 1900 << " ";
    tmpStr << std::setw(2) << std::setfill('0') << in_pTime-> tm_hour << ":";
    tmpStr << std::setw(2) << std::setfill('0') << in_pTime-> tm_min << ":";
    tmpStr << std::setw(2) << std::setfill('0') << in_pTime-> tm_sec;

    return tmpStr.str();
             
} // TestTime::FormatTimeAndDate



//************************************************************************
//************************************************************************


TTB_INLINE std::string TestTime::FormatTestDuration (
    long in_durationMs)
{
    long hours   = in_durationMs / 3600000;
    long minutes = in_durationMs % 3600000 / 60000;
    long seconds = in_durationMs % 60000 / 1000;
    long ms      = in_durationMs % 1000;

    std::ostringstream tmpStr;
    tmpStr << std::setw(2) << std::setfill('0') << hours << ":";
    tmpStr << std::setw(2) << std::setfill('0') << minutes << ":";
    tmpStr << std::setw(2) << std::setfill('0') << seconds << ",";
    tmpStr << std::setw(3) << std::setfill('0') << ms;

    return tmpStr.str();

} // TestTime::FormatTestDuration


//************************************************************************
//************************************************************************


}; // namespace TestToolBox

///\endcond
