////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \see redirecting stdout and std err from a clien process
/// (http://msdn.microsoft.com/en-us/library/windows/desktop/ms682499(v=vs.85)
///
////////////////////////////////////////////////////////////////////////////////

#pragma once

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <string>
#include <fstream>

#include "EventInfo.h"

class CaptureOutput
{
public:
	CaptureOutput(
		std::string in_fullPathToExecutable,
		bool        in_captureIsActive,
		int         in_traceLevel);

	~CaptureOutput(void);

	bool IsActive() const;
	DWORD GetWaitTime () const;
	std::string GetFileNameCapturedOutput() const;


	/// Create pipe to capture output of client process
	bool Prepare(
		int &          io_numEventsToWaitFor,
		EventInfoMap & io_eventInfos,
		HANDLE &       out_hStdOut,
		HANDLE &       out_hStdErr);

	bool ReleaseResourcesAfterStartOfExeFile();
    void HandleEventReadPipeData(HANDLE handle, EventInfoMap& eventInfoMap);
    bool HasDetectedMemLeaks (bool in_performCheck);

private:

	/// Flag indicating whether capturing of stdout is switched on
	bool m_captureIsActive;

	/// Tracelevel to control the amount of information written to stdout.
    /// 0 = no output at all, 2 = maximum output
    int m_traceLevel;

	/// Buffer size for reading from pipe
	static const int BUFSIZE = 4096;

	/// Wait timeout when waiting for remaining output written to pipe.
	/// Remark: When client process is still running the wait time will be
	/// set acccording to the specific requirements for the client process.
	static const int PIPE_TIMEOUT_MS = 5000;

	/// Handle to named pipe
	HANDLE m_hPipe;

	/// Structure to perform asynchronous reading from pipe
	OVERLAPPED m_pipeOverlapped;

	/// Handles to stdout (pipe) stream of client process
	HANDLE m_hStdOutRead;
	HANDLE m_hStdOutWrite;

	/// Output of client process will be stored within this file
	std::string  m_fileNameCapturedOutput;

	/// File stream to write to file m_fileNameCapturedOut
	std::fstream m_fsCapturedOut;

private:
		void ReleaseResourcesAfterTerminationOfExeFile();
};

