////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "Services.h"

#include <iostream>
#include <sstream>

using namespace std;

//-----------------------------------------------------------------------------

void Trim (string& str, string const & delimiters)
{
	string::size_type pos = str.find_last_not_of(delimiters);
	if(pos != string::npos)
	{
		str.erase(pos + 1);
		pos = str.find_first_not_of(delimiters);
		if(pos != string::npos) str.erase(0, pos);
	}
	else str.erase(str.begin(), str.end());

} // Trim

//-----------------------------------------------------------------------------

std::string GetSystemErrorDescription (int in_systemError)
{
    static char charBuf [4096]; 
    TCHAR* msgBufPtr = 0;

    if (::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM,
            0,
            static_cast<DWORD> (in_systemError),
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            (LPTSTR)&msgBufPtr,
            0,
            0))
    {
        char* chBufPtr = static_cast<char*> (msgBufPtr);
        if (chBufPtr)
        {
            char* chPtr;
            // Remark: The returned system message string may contain a carriage
            // return (ASCII 13) which has to be eliminated.
            if (chPtr = strchr(chBufPtr,13))
            {
                size_t len = chPtr - chBufPtr;
                strncpy_s (charBuf, chBufPtr, len);
                charBuf[len] = '\0';
            }
            else
            {
                strcpy_s (charBuf, chBufPtr);
            }
        }
        else
            strcpy_s (charBuf, "GetErrorDescription: ERROR 1");
    }
    else
        strcpy_s (charBuf, "GetErrorDescription: ERROR 2");

    if (msgBufPtr)
        LocalFree (msgBufPtr);

    return std::string(charBuf);

}

//-----------------------------------------------------------------------------

void WriteSystemErrorMsg (
	DWORD               in_error,
	std::string const & in_context)
{
	std::string errDescr = GetSystemErrorDescription(in_error);
	std::ostringstream oss;
	oss
		<< in_context << (in_context=="" ? "" : " ")
		<< "ERROR " << in_error
		<< ": " << errDescr
		<< std::endl;

	// Force synchronized output to avoid intersection with output of other processes
	printf("%s", oss.str().c_str());
}

//-----------------------------------------------------------------------------

void WriteSystemErrorMsg (
	std::string const & in_context)
{
	WriteSystemErrorMsg(GetLastError(), in_context);
}

//-----------------------------------------------------------------------------

void WriteSystemErrorMsg (
	bool                in_generateOutput,
	DWORD               in_error,
	std::string const & in_context)
{
	if (in_generateOutput)
		WriteSystemErrorMsg(in_error, in_context);
}

//-----------------------------------------------------------------------------

void WriteSystemErrorMsg (
	bool                in_generateOutput,
	std::string const & in_context)
{
	if (in_generateOutput)
		WriteSystemErrorMsg(in_context);
}

//-----------------------------------------------------------------------------

