////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <windows.h>
#include <string>

//-----------------------------------------------------------------------------

/// Helper macro to simplify exception handling
#define CATCH_ALL(M_funcName,M_successFlag) \
	catch(std::exception& exc)                                                  \
{                                                                               \
	M_successFlag = false;                                                      \
	if (m_traceLevel > 0)                                                       \
	cout << "[" << #M_funcName << "]: EXCEPTION (std::Exception): "             \
	<< exc.what() << endl;                                                      \
}                                                                               \
	catch(...)                                                                  \
{                                                                               \
	M_successFlag = false;                                                      \
	if (m_traceLevel > 0)                                                       \
	cout << "[" << #M_funcName << "]: EXCEPTION of unknown type detected!"      \
	<< endl;                                                                    \
}                                                                               \
	if (!M_successFlag)                                                         \
{                                                                               \
	if (m_traceLevel > 0)                                                       \
	cout << "[" << #M_funcName << "]: returned false!" << endl;                 \
	DisplayCallContext();                                                       \
}

//-----------------------------------------------------------------------------

/// Remove the given characters at start and end of the string
void Trim (std::string& str, std::string const & delimiters = " \t\n");

/// Generate textual description for the given system error code
std::string GetSystemErrorDescription (int in_systemError);

/// Write textual description for the givven system error code to stdout
void WriteSystemErrorMsg (
	DWORD in_error,
	std::string const & in_context = "");

/// Call get last error and write textual description to stdout
void WriteSystemErrorMsg (
	std::string const & in_context = "");

/// Conveniance functions to allow suppressing of execution depending on the given flag
void WriteSystemErrorMsg (
	bool                in_generateOutput,
	DWORD               in_error,
	std::string const & in_context = "");

/// Conveniance functions to allow suppressing of execution depending on the given flag
void WriteSystemErrorMsg (
	bool                in_generateOutput,
	std::string const & in_context = "");

//-----------------------------------------------------------------------------
