/// \file
/// \author Gerald Fahrnholz

#pragma once

#include <boost/utility.hpp>
#include <boost/thread/condition.hpp>
#include <boost/thread/mutex.hpp>

namespace TestToolBox
{
class WaitableCondition : private boost::noncopyable
{
public:
    /// Constructor
    WaitableCondition (bool in_autoReset = true);

    /// Set and reset the condition
    void Set (bool in_state);

    /// Check condition without waiting
    bool IsTrue (void);

    /// Wait until condition becomes true
    /// If the condition has already been set the function
    /// will immediately return without calling boost-wait().
    void WaitUntilTrue (void);

    /// Wait until condition becomes true (return true)
    /// or the given timeout has elapsed (return false)
    bool TimedWaitUntilTrue (long in_timeoutMs);

private:
    bool             m_autoReset;
    bool             m_condIsTrue;
    boost::mutex     m_mutex;
    boost::condition m_condition;

private:

    boost::xtime ConvertFromMilliSec (long in_timeoutMs);
};


} // namespace TestToolBox
