////////////////////////////////////////////////////////////////////////////////
///
/// \file
/// \author Gerald Fahrnholz
/// 
/// \brief
/// Main page of the generated documentation
/// 
///
/// \defgroup GrpTestUtils Using instructions
///
/// \defgroup GrpHelperApps Helper Applications
/// \brief General purpose executables which may be useful in the context of
/// testing
///
/// \defgroup GrpTestApps Test applications

/// \mainpage
///
/// <b>Overview</b>\n
/// - How to use the test utilities: \ref GrpTestUtils
///   - Accessing performance counters: \ref GrpPerformanceDataSet
///   - Accessing path informations, environment variables and command line arguments: \ref GrpEnvironment
///   - Synchronization of parallel activities: \ref GrpSynchronizer
///   - Waiting until a condition comes true: \ref GrpPollingCheck
///   - Processing general events occured during test: \ref GrpEventReceiver
///   - Processing application errors: \ref GrpErrorEvents
///   - Storing (asynchronous) test events and automatic verification: \ref GrpTestEvents
///   - Checking and comparing XML data: \ref GrpXmlCheck
/// - Helper Executables: \ref GrpHelperApps
///   - Start and control an application: \ref GrpTimedExec 
///   - Demo application with configurable runtime errors: \ref GrpDemoApp
///   - Generate a list of available performance counters: \ref GrpScanPerfCounters
/// - Test applications: \ref GrpTestApps
///   - Basic test for PerformanceDataSet: \ref GrpTestPerformanceDataSet
///   - Basic test for Environment: \ref GrpTestEnvironment
///   - Basic test for PollingCheck: \ref GrpTestPollingCheck
///   - Extensions for Boost.Test lib: \ref GrpTestExtensionsForBoostTest
///   - Basic test for XmlCheck: \ref GrpTestXmlCheckBasic
///   - Example of an unit test checking XML data with use of Boost.Test lib: \ref GrpTestXmlCheckWithBoost
///   - Example of an unit test checking XML data with use of Boost.Test and TestToolBox::TestEvents: \ref GrpTestXmlCheckWithBoostExtensions
///
/// <b>Links</b>\n
/// - <a href="http://www.gerald-fahrnholz.de/sw/display_contents.php?file_name=downloads.htm#ref--downloads--testtoolbox">Download of sources</a><br>
/// - <a href="http://www.gerald-fahrnholz.de/sw/display_contents.php?file_name=test.htm">Info about concepts for automated testing</a><br>
/// - <a href="http://www.gerald-fahrnholz.de/sw/display_contents.php?file_name=test_verification.htm">Info about checking recorded test events and comparing arbitrary expressions and values</a><br>
